@@ -86,6 +86,12 @@ struct xtvec64 {
 };
 #endif
 
+#ifdef HAVE_POSIX2008_PREADV
+#define _SYSIO_PREADV_T int
+#else
+#define _SYSIO_PREADV_T size_t
+#endif
+
 struct iovec;
 
 /*
@@ -139,7 +145,7 @@ extern ioid_t SYSIO_INTERFACE_NAME(ipread64)(int fd,
  */
 extern ssize_t SYSIO_INTERFACE_NAME(preadv)(int fd,
 					    const struct iovec *iov,
-					    size_t count,
+					    _SYSIO_PREADV_T count,
 					    off_t offset);
 
 #ifdef _LARGEFILE64_SOURCE
@@ -148,7 +154,7 @@ extern ssize_t SYSIO_INTERFACE_NAME(preadv)(int fd,
  */
 extern ssize_t SYSIO_INTERFACE_NAME(pread64v)(int fd,
 					      const struct iovec *iov, 
-					      size_t count,
+					      _SYSIO_PREADV_T count,
 					      off64_t offset);
 #endif
 
@@ -258,7 +264,7 @@ extern ioid_t SYSIO_INTERFACE_NAME(ipwrite64)(int fd,
  */
 extern ssize_t SYSIO_INTERFACE_NAME(pwritev)(int fd,
 					     const struct iovec *iov,
-					     size_t count,
+					     _SYSIO_PREADV_T count,
 					     off_t offset);
 
 #ifdef _LARGEFILE64_SOURCE
@@ -267,7 +273,7 @@ extern ssize_t SYSIO_INTERFACE_NAME(pwritev)(int fd,
  */
 extern ssize_t SYSIO_INTERFACE_NAME(pwrite64v)(int fd,
 					       const struct iovec *iov,
-					       size_t count,
+					       _SYSIO_PREADV_T count,
 					       off64_t offset);
 #endif
 
