@@ -139,7 +139,7 @@ libcfs_ioctl(struct inode *inode, struct file *file,
 	struct cfs_psdev_file	 pfile;
 	int    rc = 0;
 
-	if (current->fsuid != 0)
+	if (current_fsuid() != 0)
 		return -EACCES;
 
 	if ( _IOC_TYPE(cmd) != IOC_LIBCFS_TYPE ||
