@@ -63,7 +63,11 @@ libcfs_sock_ioctl(int cmd, unsigned long arg)
                 return rc;
         }
 
+#ifdef HAVE_SOCK_MAP_FD_2ARG
+        fd = sock_map_fd(sock,0);
+#else
         fd = sock_map_fd(sock);
+#endif
         if (fd < 0) {
                 rc = fd;
                 sock_release(sock);
@@ -679,9 +683,10 @@ libcfs_sock_connect (struct socket **sockp, int *fatal,
          * port... */
         *fatal = !(rc == -EADDRNOTAVAIL);
 
-        CDEBUG(*fatal ? D_NETERROR : D_NET,
-               "Error %d connecting %u.%u.%u.%u/%d -> %u.%u.%u.%u/%d\n", rc,
-               HIPQUAD(local_ip), local_port, HIPQUAD(peer_ip), peer_port);
+        CDEBUG_LIMIT(*fatal ? D_NETERROR : D_NET,
+                     "Error %d connecting %u.%u.%u.%u/%d -> %u.%u.%u.%u/%d\n",
+                     rc, HIPQUAD(local_ip), local_port, HIPQUAD(peer_ip),
+                     peer_port);
 
         sock_release(*sockp);
         return rc;
