@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -1332,8 +1332,9 @@ lnet_send(lnet_nid_t src_nid, lnet_msg_t *msg)
                 src_ni = lnet_nid2ni_locked(src_nid);
                 if (src_ni == NULL) {
                         LNET_UNLOCK();
-                        CERROR("Can't send to %s: src %s is not a local nid\n",
-                               libcfs_nid2str(dst_nid), libcfs_nid2str(src_nid));
+                        LCONSOLE_WARN("Can't send to %s: src %s is not a "
+                                      "local nid\n", libcfs_nid2str(dst_nid),
+                                      libcfs_nid2str(src_nid));
                         return -EINVAL;
                 }
                 LASSERT (!msg->msg_routing);
@@ -1352,8 +1353,9 @@ lnet_send(lnet_nid_t src_nid, lnet_msg_t *msg)
                         lnet_ni_decref_locked(local_ni);
                         lnet_ni_decref_locked(src_ni);
                         LNET_UNLOCK();
-                        CERROR("No route to %s via from %s\n",
-                               libcfs_nid2str(dst_nid), libcfs_nid2str(src_nid));
+                        LCONSOLE_WARN("No route to %s via from %s\n",
+                                      libcfs_nid2str(dst_nid),
+                                      libcfs_nid2str(src_nid));
                         return -EINVAL;
                 }
 
@@ -1376,8 +1378,8 @@ lnet_send(lnet_nid_t src_nid, lnet_msg_t *msg)
                 lnet_ni_decref_locked(src_ni);  /* lp has ref on src_ni; lose mine */
                 if (rc != 0) {
                         LNET_UNLOCK();
-                        CERROR("Error %d finding peer %s\n", rc,
-                               libcfs_nid2str(dst_nid));
+                        LCONSOLE_WARN("Error %d finding peer %s\n", rc,
+                                      libcfs_nid2str(dst_nid));
                         /* ENOMEM or shutting down */
                         return rc;
                 }
@@ -1402,7 +1404,8 @@ lnet_send(lnet_nid_t src_nid, lnet_msg_t *msg)
                         if (src_ni != NULL)
                                 lnet_ni_decref_locked(src_ni);
                         LNET_UNLOCK();
-                        CERROR("No route to %s\n", libcfs_id2str(msg->msg_target));
+                        LCONSOLE_WARN("No route to %s\n",
+                                      libcfs_id2str(msg->msg_target));
                         return -EHOSTUNREACH;
                 }
 
@@ -1428,9 +1431,10 @@ lnet_send(lnet_nid_t src_nid, lnet_msg_t *msg)
                                 lnet_ni_decref_locked(src_ni);
                         LNET_UNLOCK();
 
-                        CERROR("No route to %s via %s (all routers down)\n",
-                               libcfs_id2str(msg->msg_target),
-                               libcfs_nid2str(src_nid));
+                        LCONSOLE_WARN("No route to %s via %s "
+                                      "(all routers down)\n",
+                                      libcfs_id2str(msg->msg_target),
+                                      libcfs_nid2str(src_nid));
                         return -EHOSTUNREACH;
                 }
 
@@ -2437,7 +2441,7 @@ LNetPut(lnet_nid_t self, lnet_handle_md_t mdh, lnet_ack_req_t ack,
 
         rc = lnet_send(self, msg);
         if (rc != 0) {
-                CERROR("Error sending PUT to %s: %d\n",
+                CNETERR("Error sending PUT to %s: %d\n",
                        libcfs_id2str(target), rc);
                 lnet_finalize (NULL, msg, rc);
         }
@@ -2614,7 +2618,7 @@ LNetGet(lnet_nid_t self, lnet_handle_md_t mdh,
 
         rc = lnet_send(self, msg);
         if (rc < 0) {
-                CERROR("error sending GET to %s: %d\n",
+                CNETERR( "Error sending GET to %s: %d\n",
                        libcfs_id2str(target), rc);
                 lnet_finalize (NULL, msg, rc);
         }
