@@ -1465,9 +1465,7 @@ lstcon_ndlist_stat(struct list_head *ndlist,
                 return rc;
         }
 
-        timeout = (timeout > LST_TRANS_MIN_TIMEOUT) ? timeout :
-                                                      LST_TRANS_MIN_TIMEOUT;
-        lstcon_rpc_trans_postwait(trans, timeout);
+        lstcon_rpc_trans_postwait(trans, LST_VALIDATE_TIMEOUT(timeout));
 
         rc = lstcon_rpc_trans_interpreter(trans, result_up,
                                           lstcon_statrpc_readent);
@@ -1554,10 +1552,7 @@ lstcon_debug_ndlist(struct list_head *ndlist,
                 return rc;
         }
 
-        timeout = (timeout > LST_TRANS_MIN_TIMEOUT) ? timeout :
-                                                      LST_TRANS_MIN_TIMEOUT;
-
-        lstcon_rpc_trans_postwait(trans, timeout);
+        lstcon_rpc_trans_postwait(trans, LST_VALIDATE_TIMEOUT(timeout));
 
         rc = lstcon_rpc_trans_interpreter(trans, result_up,
                                           lstcon_sesrpc_readent);
@@ -1708,8 +1703,8 @@ lstcon_session_new(char *name, int key,
         if (rc != 0) {
                 lstcon_batch_t *bat;
 
-                lstcon_batch_find(LST_DEFAULT_BATCH, &bat);
-                lstcon_batch_destroy(bat);
+                if (lstcon_batch_find(LST_DEFAULT_BATCH, &bat) == 0)
+                        lstcon_batch_destroy(bat);
 
                 return rc;
         }
@@ -1984,10 +1979,10 @@ lstcon_console_fini(void)
 {
         int     i;
 
-        mutex_down(&console_session.ses_mutex);
-
         libcfs_deregister_ioctl(&lstcon_ioctl_handler);
 
+        mutex_down(&console_session.ses_mutex);
+
         srpc_shutdown_service(&lstcon_acceptor_service);
         srpc_remove_service(&lstcon_acceptor_service);
 
