@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -452,8 +452,11 @@ int ll_unregister_blkdev(unsigned int dev, const char *name)
 #endif
 
 #ifndef HAVE_SYNCHRONIZE_RCU
+/* Linux 2.6.32 provides define when !CONFIG_TREE_PREEMPT_RCU */
+#ifndef synchronize_rcu
 #define synchronize_rcu() synchronize_kernel()
 #endif
+#endif
 
 #ifdef HAVE_SECURITY_PLUG
 #define ll_vfs_rmdir(dir,entry,mnt)             vfs_rmdir(dir,entry,mnt)
@@ -658,5 +661,20 @@ static inline int ll_quota_off(struct super_block *sb, int off, int remount)
 #define blk_queue_logical_block_size(q, sz) blk_queue_hardsect_size(q, sz)
 #endif
 
+#ifdef HAVE_DQUOT_INIT
+#define ll_vfs_dq_init DQUOT_INIT
+#define ll_vfs_dq_drop DQUOT_DROP
+#else
+#define ll_vfs_dq_init vfs_dq_init
+#define ll_vfs_dq_drop vfs_dq_drop
+#endif
+
+#ifndef HAVE_REQUEST_QUEUE_LIMITS
+#define queue_max_sectors(rq)           ((rq)->max_sectors)
+#define queue_max_hw_sectors(rq)        ((rq)->max_hw_sectors)
+#define queue_max_hw_segments(rq)       ((rq)->max_hw_segments)
+#define queue_max_phys_segments(rq)     ((rq)->max_phys_segments)
+#endif
+
 #endif /* __KERNEL__ */
 #endif /* _COMPAT25_H */
