@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -124,7 +124,11 @@ struct find_param {
                         check_gid:1,
                         check_uid:1,
                         check_pool:1,
-                        exclude_pool:1;
+                        exclude_pool:1,
+                        exclude_size:1,
+                        exclude_atime:1,
+                        exclude_mtime:1,
+                        exclude_ctime:1;
 
         int     verbose;
         int     quiet;
