@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2001, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -95,11 +95,8 @@
 
 /* Defn's shared with user-space. */
 #include <lustre/lustre_user.h>
-#include <lustre_ver.h>
 #include <lustre/ll_fiemap.h>
 
-#include <libcfs/kp30.h>
-
 /*
  * this file contains all data structures used in Lustre interfaces:
  * - obdo and obd_request records
@@ -359,6 +356,9 @@ extern void lustre_swab_ptlrpc_body(struct ptlrpc_body *pb, int msgsize);
 #define OBD_CONNECT_LOV_V3    0x100000000ULL /*client supports LOV v3 EA */
 #define OBD_CONNECT_GRANT_SHRINK  0x200000000ULL /* support grant shrink */
 #define OBD_CONNECT_SKIP_ORPHAN   0x400000000ULL /* don't reuse orphan objids */
+#define OBD_CONNECT_MAX_EASIZE    0x800000000ULL /* preserved for large EA */
+#define OBD_CONNECT_FULL20       0x1000000000ULL /* it is 2.0 client */
+#define OBD_CONNECT_LAYOUTLOCK   0x2000000000ULL /* client supports layout lock */
 /* also update obd_connect_names[] for lprocfs_rd_connect_flags()
  * and lustre/utils/wirecheck.c */
 
@@ -415,13 +415,8 @@ struct obd_connect_data {
 
 extern void lustre_swab_connect(struct obd_connect_data *ocd);
 
-/* b1_6 has smaller body. The defines below is for interoperability */
-#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(2,0,0,0)
 #define PTLRPC_INTEROP_1_6      1
 #define PTLRPC_BODY_MIN_SIZE    offsetof(struct ptlrpc_body, pb_pre_versions)
-#else
-#define PTLRPC_BODY_MIN_SIZE    sizeof(struct ptlrpc_body)
-#endif
 
 /*
  * Supported checksum algorithms. Up to 32 checksum types are supported.
@@ -497,6 +492,8 @@ enum obdo_flags {
         OBD_FL_CKSUM_RSVD2  = 0x00008000, /* for future cksum types */
         OBD_FL_CKSUM_RSVD3  = 0x00010000, /* for future cksum types */
         OBD_FL_SHRINK_GRANT = 0x00020000, /* object shrink the grant */
+        OBD_FL_MMAP         = 0x00040000, /* object is mmapped on the client */
+        OBD_FL_RECOV_RESEND = 0x00080000, /* recoverable resent */
 
         OBD_FL_CKSUM_ALL    = OBD_FL_CKSUM_CRC32 | OBD_FL_CKSUM_ADLER,
 
@@ -850,7 +847,7 @@ enum {
         /*
          * This is how may FIDs may be allocated in one sequence.
          */
-        LUSTRE_SEQ_MAX_WIDTH = 0x0000000000004000ULL,
+        LUSTRE_SEQ_MAX_WIDTH = 0x0000000000020000ULL,
 };
 
 enum lu_cli_type {
@@ -2081,13 +2078,19 @@ static inline void lustre_set_wire_obdo(struct obdo *wobdo, struct obdo *lobdo)
 
 static inline void lustre_get_wire_obdo(struct obdo *lobdo, struct obdo *wobdo)
 {
-        obd_flag local_flags = lobdo->o_flags & OBD_FL_LOCAL_MASK;
+        obd_flag local_flags = 0;
+
+        if (lobdo->o_valid & OBD_MD_FLFLAGS)
+                local_flags = lobdo->o_flags & OBD_FL_LOCAL_MASK;
 
         LASSERT(!(wobdo->o_flags & OBD_FL_LOCAL_MASK));
 
         memcpy(lobdo, wobdo, sizeof(*lobdo));
-        lobdo->o_flags &= ~OBD_FL_LOCAL_MASK;
-        lobdo->o_flags |= local_flags;
+        if (local_flags != 0) {
+                lobdo->o_valid |= OBD_MD_FLFLAGS;
+                lobdo->o_flags &= ~OBD_FL_LOCAL_MASK;
+                lobdo->o_flags |= local_flags;
+        }
 }
 
 extern void lustre_swab_obdo (struct obdo *o);
@@ -2148,7 +2151,6 @@ struct qunit_data {
 #define QDATA_CLR_GRP(qdata)        ((qdata)->qd_flags &= ~LQUOTA_FLAGS_GRP)
 #define QDATA_CLR_CHANGE_QS(qdata)  ((qdata)->qd_flags &= ~LQUOTA_FLAGS_CHG_QS)
 
-#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
 /* this will be used when OBD_CONNECT_QUOTA64 is set */
 struct qunit_data_old2 {
         __u32 qd_id; /* ID appiles to (uid, gid) */
@@ -2156,16 +2158,9 @@ struct qunit_data_old2 {
                          * Block quota or file quota occupy one bit */
         __u64 qd_count; /* acquire/release count (bytes for block quota) */
 };
-#else
-#warning "remove quota code above for format absolete in new release"
-#endif
 
 extern void lustre_swab_qdata(struct qunit_data *d);
-#if LUSTRE_VERSION_CODE < OBD_OCD_VERSION(1, 9, 0, 0)
 extern void lustre_swab_qdata_old2(struct qunit_data_old2 *d);
-#else
-#warning "remove quota code above for format absolete in new release"
-#endif
 extern int quota_get_qdata(void*req, struct qunit_data *qdata,
                            int is_req, int is_exp);
 extern int quota_copy_qdata(void *request, struct qunit_data *qdata,
