@@ -129,7 +129,7 @@ struct lustre_client_ocd {
          * under ->lco_lock.
          */
         __u64      lco_flags;
-        spinlock_t lco_lock;
+        struct semaphore   lco_lock;
         struct obd_export *lco_mdc_exp;
         struct obd_export *lco_osc_exp;
 };
@@ -158,12 +158,12 @@ static inline int ll_ocd_update(struct obd_device *host,
                 flags = cli->cl_import->imp_connect_data.ocd_connect_flags;
                 CDEBUG(D_SUPER, "Changing connect_flags: "LPX64" -> "LPX64"\n",
                        lco->lco_flags, flags);
-                spin_lock(&lco->lco_lock);
+                mutex_down(&lco->lco_lock);
                 lco->lco_flags &= flags;
                 /* for each osc event update ea size */
                 if (lco->lco_osc_exp)
                         mdc_init_ea_size(lco->lco_mdc_exp, lco->lco_osc_exp);
-                spin_unlock(&lco->lco_lock);
+                mutex_up(&lco->lco_lock);
 
                 result = 0;
         } else {
