@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -184,6 +184,8 @@ int it_open_error(int phase, struct lookup_intent *it);
 void mdc_set_lock_data(__u64 *lockh, void *data, __u32 *lockbits);
 int mdc_change_cbdata(struct obd_export *exp, struct ll_fid *fid,
                       ldlm_iterator_t it, void *data);
+int mdc_find_cbdata(struct obd_export *exp, struct ll_fid *fid,
+                    ldlm_iterator_t it, void *data);
 int mdc_revalidate_lock(struct obd_export *exp,
                         struct lookup_intent *it,
                         struct ll_fid *fid);
