@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -349,7 +349,9 @@ struct filter_obd {
         unsigned long        fo_read_cache:1,         /* read-only cache */
                              fo_writethrough_cache:1, /* writetrhough cache */
                              fo_syncjournal:1,      /* sync journal on writes */
-                             fo_raid_degraded:1;    /* RAID device degraded */
+                             fo_sync_lock_cancel:2, /* sync on lock cancel */
+                             fo_raid_degraded:1,    /* RAID device degraded */
+                             fo_mds_ost_sync:1; /**< MDS-OST orphan recovery*/
 
         struct obd_import   *fo_mdc_imp;
         struct obd_uuid      fo_mdc_uuid;
@@ -401,6 +403,14 @@ struct filter_obd {
 #define OSC_MAX_DIRTY_MB_MAX   2048     /* arbitrary, but < MAX_LONG bytes */
 #define OSC_DEFAULT_RESENDS      10
 
+/* possible values for fo_sync_lock_cancel */
+enum {
+        NEVER_SYNC_ON_CANCEL = 0,
+        BLOCKING_SYNC_ON_CANCEL = 1,
+        ALWAYS_SYNC_ON_CANCEL = 2,
+        NUM_SYNC_ON_CANCEL_STATES
+};
+
 #define MDC_MAX_RIF_DEFAULT       8
 #define MDC_MAX_RIF_MAX         512
 
@@ -499,6 +509,8 @@ struct client_obd {
         struct list_head         cl_loi_read_list;
         int                      cl_r_in_flight;
         int                      cl_w_in_flight;
+        int                      cl_dio_r_in_flight;
+        int                      cl_dio_w_in_flight;
         /* just a sum of the loi/lop pending numbers to be exported by /proc */
         int                      cl_pending_w_pages;
         int                      cl_pending_r_pages;
@@ -553,6 +565,7 @@ struct client_obd {
 #define CL_NOT_QUOTACHECKED 1   /* client->cl_qchk_stat init value */
 
 struct mgs_obd {
+        struct obd_device_target         mgs_obt;
         struct ptlrpc_service           *mgs_service;
         struct vfsmount                 *mgs_vfsmnt;
         struct super_block              *mgs_sb;
@@ -657,7 +670,6 @@ struct ost_obd {
         struct ptlrpc_service *ost_create_service;
         struct ptlrpc_service *ost_io_service;
         struct semaphore       ost_health_sem;
-        int                    ost_sync_on_lock_cancel;
 };
 
 struct echo_client_obd {
@@ -1108,6 +1120,7 @@ enum obd_cleanup_stage {
 #define KEY_MAX_EASIZE          "max_ea_size"
 #define KEY_FIEMAP              "fiemap"
 #define KEY_CONNECT_FLAG        "connect_flags"
+#define KEY_SYNC_LOCK_CANCEL    "sync_lock_cancel"
 /* XXX unused */
 #define KEY_ASYNC               "async"
 #define KEY_CAPA_KEY            "capa_key"
@@ -1259,6 +1272,8 @@ struct obd_ops {
                        struct lustre_handle *lockh, int *n_matches);
         int (*o_change_cbdata)(struct obd_export *, struct lov_stripe_md *,
                                ldlm_iterator_t it, void *data);
+        int (*o_find_cbdata)(struct obd_export *, struct lov_stripe_md *,
+                             ldlm_iterator_t it, void *data);
         int (*o_cancel)(struct obd_export *, struct lov_stripe_md *md,
                         __u32 mode, struct lustre_handle *, int flags,
                         obd_off end);
@@ -1275,6 +1290,7 @@ struct obd_ops {
         int (*o_llog_init)(struct obd_device *obd, struct obd_device *disk_obd,
                            int *idx);
         int (*o_llog_finish)(struct obd_device *obd, int count);
+        int (*o_llog_connect)(struct obd_export *, struct llogd_conn_body *);
 
         /* metadata-only methods */
         int (*o_pin)(struct obd_export *, struct ll_fid *,
