@@ -52,16 +52,15 @@
  */
 struct ldlm_resource * lock_res_and_lock(struct ldlm_lock *lock)
 {
-        struct ldlm_resource *res = lock->l_resource;
-
-        if (ns_is_server(res->lr_namespace)) {
-                /* on server-side resource of lock doesn't change */
-                lock_res(res);
-                return res;
-        } 
+        struct ldlm_resource *res = NULL;
 
         spin_lock(&lock->l_lock);
         res = lock->l_resource;
+
+        if (ns_is_server(res->lr_namespace))
+                /* on server-side resource of lock doesn't change */
+                spin_unlock(&lock->l_lock);
+
         lock_res(res);
         return res;
 }
