@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -267,6 +267,8 @@ int client_obd_setup(struct obd_device *obddev, obd_count len, void *buf)
         client_obd_list_lock_init(&cli->cl_loi_list_lock);
         cli->cl_r_in_flight = 0;
         cli->cl_w_in_flight = 0;
+        cli->cl_dio_r_in_flight = 0;
+        cli->cl_dio_w_in_flight = 0;
         spin_lock_init(&cli->cl_read_rpc_hist.oh_lock);
         spin_lock_init(&cli->cl_write_rpc_hist.oh_lock);
         spin_lock_init(&cli->cl_read_page_hist.oh_lock);
@@ -551,9 +553,6 @@ int server_disconnect_export(struct obd_export *exp)
         }
         spin_unlock(&exp->exp_lock);
 
-        /* release nid stat refererence */
-        lprocfs_exp_cleanup(exp);
-
         RETURN(rc);
 }
 
@@ -624,7 +623,7 @@ int target_recovery_check_and_stop(struct obd_device *obd)
 {
         int abort_recovery = 0;
 
-        if (obd->obd_stopping)
+        if (obd->obd_stopping || !obd->obd_recovering)
                 return 1;
 
         spin_lock_bh(&obd->obd_processing_task_lock);
@@ -633,7 +632,7 @@ int target_recovery_check_and_stop(struct obd_device *obd)
         spin_unlock_bh(&obd->obd_processing_task_lock);
         if (!abort_recovery)
                 return 0;
-        /** check is fs version-capable */
+        /** check if fs version-capable */
         if (target_fs_version_capable(obd)) {
                 class_handle_stale_exports(obd);
         } else {
@@ -1172,7 +1171,9 @@ static void target_request_copy_get(struct ptlrpc_request *req)
         /* mark that request is in recovery queue, so request handler will not
          * drop rpc count in export, bug 19870*/
         LASSERT(!req->rq_copy_queued);
+        spin_lock(&req->rq_lock);
         req->rq_copy_queued = 1;
+        spin_unlock(&req->rq_lock);
         /* increase refcount to keep request in queue */
         atomic_inc(&req->rq_refcount);
         /* release service thread while request is queued
@@ -1262,7 +1263,7 @@ static void abort_recovery_queue(struct obd_device *obd)
         list_for_each_entry_safe(req, n, &abort_list, rq_list) {
                 target_exp_dequeue_req_replay(req);
                 list_del_init(&req->rq_list);
-                DEBUG_REQ(D_ERROR, req, "aborted:");
+                DEBUG_REQ(D_ERROR, req, "%s: aborted:", obd->obd_name);
                 req->rq_status = -ENOTCONN;
                 req->rq_type = PTL_RPC_MSG_ERR;
                 rc = lustre_pack_reply(req, 1, NULL, NULL);
@@ -1503,11 +1504,22 @@ static int check_for_next_transno(struct obd_device *obd)
         __u64 next_transno, req_transno;
 
         if (obd->obd_stopping) {
-                CDEBUG(D_HA, "waking for stooping device\n");
+                CDEBUG(D_HA, "waking for stopping device\n");
                 return 1;
         }
 
         spin_lock_bh(&obd->obd_processing_task_lock);
+        if (obd->obd_abort_recovery) {
+                CDEBUG(D_HA, "waking for aborted recovery\n");
+                spin_unlock_bh(&obd->obd_processing_task_lock);
+                return 1;
+        } else if (!obd->obd_recovering) {
+                CDEBUG(D_HA, "waking for completed recovery (?)\n");
+                spin_unlock_bh(&obd->obd_processing_task_lock);
+                return 1;
+        }
+
+        LASSERT(!list_empty(&obd->obd_recovery_queue));
         req = list_entry(obd->obd_recovery_queue.next,
                          struct ptlrpc_request, rq_list);
         max = obd->obd_max_recoverable_clients;
@@ -1522,20 +1534,15 @@ static int check_for_next_transno(struct obd_device *obd)
                "queue_len: %d, req_transno: "LPU64", next_transno: "LPU64"\n",
                max, connected, obd->obd_delayed_clients, completed, queue_len,
                req_transno, next_transno);
-        if (obd->obd_abort_recovery) {
-                CDEBUG(D_HA, "waking for aborted recovery\n");
-                wake_up = 1;
-        } else if (!obd->obd_recovering) {
-                CDEBUG(D_HA, "waking for completed recovery (?)\n");
-                wake_up = 1;
-        } else if (req_transno == next_transno) {
+        if (req_transno == next_transno) {
                 CDEBUG(D_HA, "waking for next ("LPD64")\n", next_transno);
                 wake_up = 1;
         } else if (queue_len == obd->obd_recoverable_clients) {
                 CDEBUG(D_ERROR,
-                       "waking for skipped transno (skip: "LPD64
+                       "%s: waking for skipped transno (skip: "LPD64
                        ", ql: %d, comp: %d, conn: %d, next: "LPD64")\n",
-                       next_transno, queue_len, completed, max, req_transno);
+                       obd->obd_name, next_transno, queue_len, completed, max,
+                       req_transno);
                 obd->obd_next_recovery_transno = req_transno;
                 wake_up = 1;
         }
@@ -1552,22 +1559,34 @@ static void process_recovery_queue(struct obd_device *obd)
 
         for (;;) {
                 spin_lock_bh(&obd->obd_processing_task_lock);
-                LASSERT(obd->obd_processing_task == cfs_curproc_pid());
+
+                if (!obd->obd_recovering) {
+                        spin_unlock_bh(&obd->obd_processing_task_lock);
+                        EXIT;
+                        return;
+                }
+
+                LASSERTF(obd->obd_processing_task == cfs_curproc_pid(),
+                         "%s: invalid pid in obd_processing_task (%d != %d)\n",
+                         obd->obd_name, obd->obd_processing_task,
+                         cfs_curproc_pid());
                 req = list_entry(obd->obd_recovery_queue.next,
                                  struct ptlrpc_request, rq_list);
 
                 if (lustre_msg_get_transno(req->rq_reqmsg) !=
                     obd->obd_next_recovery_transno) {
                         spin_unlock_bh(&obd->obd_processing_task_lock);
-                        CDEBUG(D_HA, "Waiting for transno "LPD64" (1st is "
-                               LPD64", x"LPU64")\n",
+                        CDEBUG(D_HA, "%s: waiting for transno "LPD64" (1st is "
+                               LPD64", x"LPU64")\n", obd->obd_name,
                                obd->obd_next_recovery_transno,
                                lustre_msg_get_transno(req->rq_reqmsg),
                                req->rq_xid);
                         l_wait_event(obd->obd_next_transno_waitq,
                                      check_for_next_transno(obd), &lwi);
-                        if (target_recovery_check_and_stop(obd))
+                        if (target_recovery_check_and_stop(obd)) {
+                                EXIT;
                                 return;
+                        }
                         continue;
                 }
                 list_del_init(&req->rq_list);
@@ -1622,6 +1641,12 @@ int target_queue_recovery_request(struct ptlrpc_request *req,
         }
 
         spin_lock_bh(&obd->obd_processing_task_lock);
+
+        if (!obd->obd_recovering) {
+                spin_unlock_bh(&obd->obd_processing_task_lock);
+                RETURN(0);
+        }
+
         /* If we're processing the queue, we want don't want to queue this
          * message.
          *
@@ -1647,7 +1672,8 @@ int target_queue_recovery_request(struct ptlrpc_request *req,
 
         if (target_exp_enqueue_req_replay(req)) {
                 spin_unlock_bh(&obd->obd_processing_task_lock);
-                DEBUG_REQ(D_ERROR, req, "dropping resent queued req");
+                DEBUG_REQ(D_ERROR, req, "%s: dropping resent queued req",
+                                        obd->obd_name);
                 RETURN(0);
         }
 
@@ -1665,8 +1691,9 @@ int target_queue_recovery_request(struct ptlrpc_request *req,
                 if (unlikely(lustre_msg_get_transno(reqiter->rq_reqmsg) ==
                              transno)) {
                         spin_unlock_bh(&obd->obd_processing_task_lock);
-                        DEBUG_REQ(D_ERROR, req, "dropping replay: transno "
-                                  "has been claimed by another client");
+                        DEBUG_REQ(D_ERROR, req, "%s: dropping replay: transno "
+                                  "has been claimed by another client",
+                                  obd->obd_name);
                         target_exp_dequeue_req_replay(req);
                         RETURN(0);
                 }
