@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -239,6 +239,9 @@ static inline void ldlm_pool_recalc_grant_plan(struct ldlm_pool *pl)
         grant_step = ldlm_pool_t2gsp(pl->pl_recalc_period);
         grant_step = ((limit - granted) * grant_step) / 100;
         pl->pl_grant_plan = granted + grant_step;
+        limit = (limit * 5) >> 2;
+        if (pl->pl_grant_plan > limit)
+                pl->pl_grant_plan = limit;
 }
 
 /**
@@ -274,10 +277,6 @@ static inline void ldlm_pool_recalc_slv(struct ldlm_pool *pl)
          */
         slv_factor = (grant_usage << LDLM_POOL_SLV_SHIFT);
         do_div(slv_factor, limit);
-        if (2 * abs(granted - limit) > limit) {
-                slv_factor *= slv_factor;
-                slv_factor = dru(slv_factor, LDLM_POOL_SLV_SHIFT, round_up);
-        }
         slv = slv * slv_factor;
         slv = dru(slv, LDLM_POOL_SLV_SHIFT, round_up);
 
@@ -1063,7 +1062,8 @@ static int ldlm_pools_shrink(ldlm_side_t client, int nr,
         int total = 0, cached = 0, nr_ns;
         struct ldlm_namespace *ns;
 
-        if (nr != 0 && !(gfp_mask & __GFP_FS))
+        if (client == LDLM_NAMESPACE_CLIENT && nr != 0 &&
+            !(gfp_mask & __GFP_FS))
                 return -1;
 
         if (nr != 0)
