@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -111,6 +111,38 @@ int ll_dcompare(struct dentry *parent, struct qstr *d_name, struct qstr *name)
         RETURN(0);
 }
 
+static inline int return_if_equal(struct ldlm_lock *lock, void *data)
+{
+        if (lock->l_flags & LDLM_FL_CANCELING)
+                return LDLM_ITER_CONTINUE;
+        return LDLM_ITER_STOP;
+}
+
+/* find any ldlm lock of the inode in mdc and lov
+ * return 0    not find
+ *        1    find one
+ *      < 0    error */
+int find_cbdata(struct inode *inode)
+{
+        struct ll_fid fid;
+        struct ll_inode_info *lli = ll_i2info(inode);
+        struct ll_sb_info *sbi = ll_i2sbi(inode);
+        int rc = 0;
+        ENTRY;
+
+        LASSERT(inode);
+        ll_inode2fid(&fid, inode);
+        rc = mdc_find_cbdata(sbi->ll_mdc_exp, &fid, return_if_equal, NULL);
+        if (rc != 0)
+                RETURN(rc);
+
+        if (lli->lli_smd)
+                rc = obd_find_cbdata(sbi->ll_osc_exp, lli->lli_smd,
+                                     return_if_equal, NULL);
+
+        RETURN(rc);
+}
+
 /* should NOT be called with the dcache lock, see fs/dcache.c */
 static int ll_ddelete(struct dentry *de)
 {
@@ -123,6 +155,15 @@ static int ll_ddelete(struct dentry *de)
                d_unhashed(de) ? "" : "hashed,",
                list_empty(&de->d_subdirs) ? "" : "subdirs");
 
+        /* if not ldlm lock for this inode, set i_nlink to 0 so that
+         * this inode can be recycled later b=20433 */
+        LASSERT(atomic_read(&de->d_count) == 0);
+        if (de->d_inode && !find_cbdata(de->d_inode))
+                de->d_inode->i_nlink = 0;
+
+        if (de->d_flags & DCACHE_LUSTRE_INVALID)
+                RETURN(1);
+
         RETURN(0);
 }
 
@@ -345,7 +386,7 @@ int ll_revalidate_it(struct dentry *de, int lookup_flags,
         struct lookup_intent lookup_it = { .it_op = IT_LOOKUP };
         struct obd_export *exp;
         struct inode *parent = de->d_parent->d_inode;
-        int first = 0, rc;
+        int first = -1, rc;
 
         ENTRY;
         CDEBUG(D_VFSTRACE, "VFS Op:name=%s,intent=%s\n", de->d_name.name,
@@ -365,7 +406,7 @@ int ll_revalidate_it(struct dentry *de, int lookup_flags,
                         RETURN(0);
 #endif
 
-                rc = ll_have_md_lock(parent, MDS_INODELOCK_UPDATE);
+                rc = ll_have_md_lock(parent, MDS_INODELOCK_UPDATE, LCK_MINMODE);
                 GOTO(out_sa, rc);
         }
 
@@ -418,7 +459,7 @@ int ll_revalidate_it(struct dentry *de, int lookup_flags,
                         och_usecount = &lli->lli_open_fd_read_count;
                 }
                 /* Check for the proper lock. */
-                if (!ll_have_md_lock(inode, MDS_INODELOCK_LOOKUP))
+                if (!ll_have_md_lock(inode, MDS_INODELOCK_LOOKUP, LCK_MINMODE))
                         goto do_lock;
                 down(&lli->lli_och_sem);
                 if (*och_p) { /* Everything is open already, do nothing */
@@ -757,10 +798,19 @@ out_it:
 }
 #endif
 
+void ll_d_iput(struct dentry *de, struct inode *inode)
+{
+        LASSERT(inode);
+        if (inode && !find_cbdata(inode))
+                inode->i_nlink = 0;
+        iput(inode);
+}
+
 struct dentry_operations ll_d_ops = {
         .d_revalidate = ll_revalidate_nd,
         .d_release = ll_release,
         .d_delete = ll_ddelete,
+        .d_iput   = ll_d_iput,
 #ifdef DCACHE_LUSTRE_INVALID
         .d_compare = ll_dcompare,
 #endif
