@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -185,7 +185,7 @@ struct ll_inode_info {
         void                   *lli_opendir_key;
         struct ll_statahead_info *lli_sai;
         struct rw_semaphore     lli_truncate_rwsem;
-        /* the most recent attributes from mds, it is used for timestampts
+        /* the most recent attributes from mds, it is used for timestamps
          * only so far */
         struct ost_lvb         lli_lvb;
         struct inode            lli_vfs_inode;
@@ -303,11 +303,12 @@ enum stats_track_type {
 #define LL_SBI_LRU_RESIZE       0x80 /* support lru resize */
 #define LL_SBI_LLITE_CHECKSUM  0x100 /* checksum each page in memory */
 #define LL_SBI_LAZYSTATFS      0x200 /* lazystatfs mount option */
+#define LL_SBI_32BIT_API       0x400 /* generate 32 bit inodes. */
 
 /* default value for ll_sb_info->contention_time */
 #define SBI_DEFAULT_CONTENTION_SECONDS     60
 /* default value for lockless_truncate_enable */
-#define SBI_DEFAULT_LOCKLESS_TRUNCATE_ENABLE 1
+#define SBI_DEFAULT_LOCKLESS_TRUNCATE_ENABLE 0 /* see bug 23175 */
 /* default value for ll_direct_io_default */
 #define SBI_DEFAULT_DIRECT_IO_DEFAULT 0
 
@@ -375,10 +376,6 @@ struct ll_sb_info {
         unsigned int              ll_namelen;
         struct file_operations   *ll_fop;
 
-#ifdef HAVE_EXPORT___IGET
-        struct list_head          ll_deathrow; /* inodes to be destroyed (b1443) */
-        spinlock_t                ll_deathrow_lock;
-#endif
         /* =0 - hold lock over whole read/write
          * >0 - max. chunk to be read/written w/o lock re-acquiring */
         unsigned long             ll_max_rw_chunk;
@@ -639,6 +636,15 @@ void ll_ra_read_init(struct file *f, struct ll_ra_read *rar,
 void ll_ra_read_ex(struct file *f, struct ll_ra_read *rar);
 struct ll_ra_read *ll_ra_read_get(struct file *f);
 
+static inline int ll_need_32bit_api(struct ll_sb_info *sbi)
+{
+#if BITS_PER_LONG == 32
+        return 1;
+#else
+        return unlikely(cfs_curproc_is_32bit() || (sbi->ll_flags & LL_SBI_32BIT_API));
+#endif
+}
+
 /* llite/lproc_llite.c */
 #ifdef LPROCFS
 int lprocfs_register_mountpoint(struct proc_dir_entry *parent,
@@ -743,7 +749,7 @@ extern struct file_operations ll_file_operations_flock;
 extern struct file_operations ll_file_operations_noflock;
 extern struct inode_operations ll_file_inode_operations;
 extern int ll_inode_revalidate_it(struct dentry *, struct lookup_intent *);
-extern int ll_have_md_lock(struct inode *inode, __u64 bits);
+extern int ll_have_md_lock(struct inode *inode, __u64 bits, ldlm_mode_t l_req_mode);
 int ll_region_mapped(unsigned long addr, size_t count);
 int ll_extent_lock(struct ll_file_data *, struct inode *,
                    struct lov_stripe_md *, int mode, ldlm_policy_data_t *,
@@ -1189,8 +1195,8 @@ enum llioc_iter ll_iocontrol_call(struct inode *inode, struct file *file,
 void *ll_iocontrol_register(llioc_callback_t cb, int count, unsigned int *cmd);
 void ll_iocontrol_unregister(void *magic);
 
-ino_t ll_fid_build_ino(struct ll_sb_info *sbi,
-                       struct ll_fid *fid);
+__u64 ll_fid_build_ino(const struct ll_fid *fid);
+__u32 ll_fid_build_ino32(const struct ll_fid *fid);
 __u32 ll_fid_build_gen(struct ll_sb_info *sbi,
                        struct ll_fid *fid);
 
