@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -199,10 +199,6 @@ static struct ll_sb_info *ll_init_sbi(void)
         sbi->ll_flags |= LL_SBI_LRU_RESIZE;
 #endif
 
-#ifdef HAVE_EXPORT___IGET
-        INIT_LIST_HEAD(&sbi->ll_deathrow);
-        spin_lock_init(&sbi->ll_deathrow_lock);
-#endif
         for (i = 0; i <= LL_PROCESS_HIST_MAX; i++) {
                 spin_lock_init(&sbi->ll_rw_extents_info.pp_extents[i].pp_r_hist.oh_lock);
                 spin_lock_init(&sbi->ll_rw_extents_info.pp_extents[i].pp_w_hist.oh_lock);
@@ -416,10 +412,6 @@ static int client_common_fill_super(struct super_block *sb,
         obd->obd_upcall.onu_upcall = ll_ocd_update;
         data->ocd_brw_size = PTLRPC_MAX_BRW_PAGES << CFS_PAGE_SHIFT;
 
-        /* ask lov to generate OBD_NOTIFY_CREATE events for already registered
-         * targets */
-        obd_notify(obd, NULL, OBD_NOTIFY_CREATE, NULL);
-
         obd_register_lock_cancel_cb(obd, ll_extent_lock_cancel_cb);
         obd_register_page_removal_cb(obd, ll_page_removal_cb, ll_pin_extent_cb);
 
@@ -501,7 +493,7 @@ static int client_common_fill_super(struct super_block *sb,
         }
 
         LASSERT(sbi->ll_rootino != 0);
-        root = ll_iget(sb, ll_fid_build_ino(sbi, &rootfid), &md);
+        root = ll_iget(sb, ll_fid_build_ino(&rootfid), &md);
 
         ptlrpc_req_finished(request);
 
@@ -625,101 +617,6 @@ void lustre_dump_dentry(struct dentry *dentry, int recur)
         }
 }
 
-#ifdef HAVE_EXPORT___IGET
-static void prune_dir_dentries(struct inode *inode)
-{
-        struct dentry *dentry, *prev = NULL;
-
-        /* due to lustre specific logic, a directory
-         * can have few dentries - a bug from VFS POV */
-restart:
-        spin_lock(&dcache_lock);
-        if (!list_empty(&inode->i_dentry)) {
-                dentry = list_entry(inode->i_dentry.prev,
-                                    struct dentry, d_alias);
-                /* in order to prevent infinite loops we
-                 * break if previous dentry is busy */
-                if (dentry != prev) {
-                        prev = dentry;
-                        dget_locked(dentry);
-                        spin_unlock(&dcache_lock);
-
-                        /* try to kill all child dentries */
-                        shrink_dcache_parent(dentry);
-                        dput(dentry);
-
-                        /* now try to get rid of current dentry */
-                        d_prune_aliases(inode);
-                        goto restart;
-                }
-        }
-        spin_unlock(&dcache_lock);
-}
-
-static void prune_deathrow_one(struct ll_inode_info *lli)
-{
-        struct inode *inode = ll_info2i(lli);
-
-        /* first, try to drop any dentries - they hold a ref on the inode */
-        if (S_ISDIR(inode->i_mode))
-                prune_dir_dentries(inode);
-        else
-                d_prune_aliases(inode);
-
-
-        /* if somebody still uses it, leave it */
-        LASSERT(atomic_read(&inode->i_count) > 0);
-        if (atomic_read(&inode->i_count) > 1)
-                goto out;
-
-        CDEBUG(D_INODE, "inode %lu/%u(%d) looks a good candidate for prune\n",
-               inode->i_ino,inode->i_generation, atomic_read(&inode->i_count));
-
-        /* seems nobody uses it anymore */
-        inode->i_nlink = 0;
-
-out:
-        iput(inode);
-        return;
-}
-
-static void prune_deathrow(struct ll_sb_info *sbi, int try)
-{
-        struct ll_inode_info *lli;
-        int empty;
-
-        do {
-                if (need_resched() && try)
-                        break;
-
-                if (try) {
-                        if (!spin_trylock(&sbi->ll_deathrow_lock))
-                                break;
-                } else {
-                        spin_lock(&sbi->ll_deathrow_lock);
-                }
-
-                empty = 1;
-                lli = NULL;
-                if (!list_empty(&sbi->ll_deathrow)) {
-                        lli = list_entry(sbi->ll_deathrow.next,
-                                         struct ll_inode_info,
-                                         lli_dead_list);
-                        list_del_init(&lli->lli_dead_list);
-                        if (!list_empty(&sbi->ll_deathrow))
-                                empty = 0;
-                }
-                spin_unlock(&sbi->ll_deathrow_lock);
-
-                if (lli)
-                        prune_deathrow_one(lli);
-
-        } while (empty == 0);
-}
-#else /* !HAVE_EXPORT___IGET */
-#define prune_deathrow(sbi, try) do {} while (0)
-#endif /* HAVE_EXPORT___IGET */
-
 void client_common_put_super(struct super_block *sb)
 {
         struct ll_sb_info *sbi = ll_s2sbi(sb);
@@ -729,9 +626,6 @@ void client_common_put_super(struct super_block *sb)
 
         lprocfs_unregister_mountpoint(sbi);
 
-        /* destroy inodes in deathrow */
-        prune_deathrow(sbi, 0);
-
         list_del(&sbi->ll_conn_chain);
 
         /* callbacks is cleared after disconnect each target */
@@ -887,6 +781,11 @@ static int ll_options(char *options, int *flags)
                         *flags &= ~tmp;
                         goto next;
                 }
+                tmp = ll_set_opt("32bitapi", s1, LL_SBI_32BIT_API);
+                if (tmp) {
+                        *flags |= tmp;
+                        goto next;
+                }
                 LCONSOLE_ERROR_MSG(0x152, "Unknown option '%s', won't mount.\n",
                                    s1);
                 RETURN(-EINVAL);
@@ -1097,6 +996,10 @@ out:
 }
 /* end COMPAT_146 */
 
+#ifdef HAVE_NEW_BACKING_DEV_INFO
+static atomic_t ll_bdi_num = ATOMIC_INIT(0);
+#endif
+
 int ll_fill_super(struct super_block *sb)
 {
         struct lustre_profile *lprof = NULL;
@@ -1225,6 +1128,21 @@ int ll_fill_super(struct super_block *sb)
 
         /* connections, registrations, sb setup */
         err = client_common_fill_super(sb, mdc, osc);
+        if (err)
+                GOTO(out_free, err);
+
+#ifdef HAVE_BDI_INIT
+        err = bdi_init(&lsi->bdi);
+        if (err)
+                GOTO(out_free, err);
+#endif
+#ifdef HAVE_NEW_BACKING_DEV_INFO
+        lsi->bdi.name = "lustre";
+        lsi->bdi.capabilities = BDI_CAP_MAP_COPY;
+        err = bdi_register(&lsi->bdi, NULL, "lustre-%d",
+                                atomic_inc_return(&ll_bdi_num));
+        sb->s_bdi = &lsi->bdi;
+#endif
 
 out_free:
         if (save && lprof)
@@ -1288,6 +1206,16 @@ void ll_put_super(struct super_block *sb)
         if (profilenm)
                 class_del_profile(profilenm);
 
+#ifdef HAVE_NEW_BACKING_DEV_INFO
+        if (lsi->bdi.wb_cnt > 0) {
+#endif
+#ifdef HAVE_BDI_INIT
+                bdi_destroy(&lsi->bdi);
+#endif
+#ifdef HAVE_NEW_BACKING_DEV_INFO
+        }
+#endif
+
         ll_free_sbi(sb);
         lsi->lsi_llsbi = NULL;
 
@@ -1305,7 +1233,7 @@ int ll_shrink_cache(int nr_to_scan, SHRINKER_MASK_T gfp_mask)
         struct ll_sb_info *sbi;
         int count = 0;
 
-        if ((nr_to_scan != 0) && (gfp_mask & __GFP_FS))
+        if ((nr_to_scan != 0) && !(gfp_mask & __GFP_FS))
                 return -1;
 
         /* don't race with umount */
@@ -1412,12 +1340,6 @@ void ll_clear_inode(struct inode *inode)
 
         lli->lli_inode_magic = LLI_INODE_DEAD;
 
-#ifdef HAVE_EXPORT___IGET
-        spin_lock(&sbi->ll_deathrow_lock);
-        list_del_init(&lli->lli_dead_list);
-        spin_unlock(&sbi->ll_deathrow_lock);
-#endif
-
         EXIT;
 }
 static int ll_setattr_do_truncate(struct inode *inode, loff_t new_size)
@@ -1653,6 +1575,7 @@ int ll_setattr_raw(struct inode *inode, struct iattr *attr)
                         if (LTIME_S(attr->ia_mtime) < LTIME_S(attr->ia_ctime)){
                                 struct ost_lvb xtimes;
 
+                                UNLOCK_INODE_MUTEX(inode);
                                 /* setting mtime to past is performed under PW
                                  * EOF extent lock */
                                 oinfo->oi_policy.l_extent.start = 0;
@@ -1660,7 +1583,9 @@ int ll_setattr_raw(struct inode *inode, struct iattr *attr)
                                 rc = ll_extent_lock(NULL, inode, lsm, LCK_PW,
                                                     &oinfo->oi_policy,
                                                     &lockh, 0);
+                                LOCK_INODE_MUTEX(inode);
                                 if (rc) {
+                                        OBDO_FREE(oa);
                                         OBD_FREE_PTR(oinfo);
                                         RETURN(rc);
                                 }
@@ -1997,7 +1922,7 @@ void ll_update_inode(struct inode *inode, struct lustre_md *md)
         }
 #endif
 
-        inode->i_ino = ll_fid_build_ino(sbi, &body->fid1);
+        inode->i_ino = ll_fid_build_ino(&body->fid1);
         inode->i_generation = ll_fid_build_gen(sbi, &body->fid1);
         *ll_inode_lu_fid(inode) = *((struct lu_fid*)&md->body->fid1);
 
@@ -2061,15 +1986,6 @@ void ll_update_inode(struct inode *inode, struct lustre_md *md)
         EXIT;
 }
 
-static struct backing_dev_info ll_backing_dev_info = {
-        .ra_pages       = 0,    /* No readahead */
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12))
-        .capabilities   = 0,    /* Does contribute to dirty memory */
-#else
-        .memory_backed  = 0,    /* Does contribute to dirty memory */
-#endif
-};
-
 void ll_read_inode2(struct inode *inode, void *opaque)
 {
         struct lustre_md *md = opaque;
@@ -2095,6 +2011,9 @@ void ll_read_inode2(struct inode *inode, void *opaque)
 
         /* OIDEBUG(inode); */
 
+        /* initializing backing dev info. */
+        inode->i_mapping->backing_dev_info = &(s2lsi(inode->i_sb)->bdi);
+
         if (S_ISREG(inode->i_mode)) {
                 struct ll_sb_info *sbi = ll_i2sbi(inode);
                 inode->i_op = &ll_file_inode_operations;
@@ -2113,8 +2032,6 @@ void ll_read_inode2(struct inode *inode, void *opaque)
                 inode->i_op = &ll_special_inode_operations;
                 init_special_inode(inode, inode->i_mode,
                                    kdev_t_to_nr(inode->i_rdev));
-                /* initializing backing dev info. */
-                inode->i_mapping->backing_dev_info = &ll_backing_dev_info;
                 EXIT;
         }
 }
@@ -2328,7 +2245,6 @@ int ll_prep_inode(struct obd_export *exp, struct inode **inode,
 
         LASSERT(*inode || sb);
         sbi = sb ? ll_s2sbi(sb) : ll_i2sbi(*inode);
-        prune_deathrow(sbi, 1);
 
         rc = mdc_req2lustre_md(req, offset, exp, &md);
         if (rc)
@@ -2341,7 +2257,7 @@ int ll_prep_inode(struct obd_export *exp, struct inode **inode,
                 /** hashing VFS inode by FIDs.
                  * IGIF will be used for for compatibility if needed.
                  */
-                *inode =ll_iget(sb, ll_fid_build_ino(sbi, &md.body->fid1), &md);
+                *inode =ll_iget(sb, ll_fid_build_ino(&md.body->fid1), &md);
                 if (*inode == NULL || is_bad_inode(*inode)) {
                         mdc_free_lustre_md(exp, &md);
                         rc = -ENOMEM;
