@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -492,6 +492,15 @@ void ll_inode_fill_obdo(struct inode *inode, int cmd, struct obdo *oa)
         }
 
         obdo_from_inode(oa, inode, valid_flags);
+        /* Bug11742 - set the OBD_FL_MMAP flag for memory mapped files */
+        if (atomic_read(&(ll_i2info(inode)->lli_mmap_cnt)) != 0) {
+                if (!(oa->o_valid & OBD_MD_FLFLAGS)) {
+                        oa->o_valid |= OBD_MD_FLFLAGS;
+                        oa->o_flags = OBD_FL_MMAP;
+                } else {
+                        oa->o_flags |= OBD_FL_MMAP;
+                }
+        }
 }
 
 static void ll_ap_fill_obdo(void *data, int cmd, struct obdo *oa)
@@ -913,8 +922,17 @@ static struct ll_async_page *llap_from_page_with_lockh(struct page *page,
                                page, csum);
                 } else {
                         /* origin == LLAP_ORIGIN_WRITEPAGE */
-                        LL_CDEBUG_PAGE(D_ERROR, page, "old cksum %x != new "
-                                       "%x!\n", llap->llap_checksum, csum);
+                        if (!atomic_read(&(ll_i2info(inode)->lli_mmap_cnt))) {
+			        LL_CDEBUG_PAGE(D_ERROR, page,
+                                               "old cksum %x != new %x!\n",
+                                               llap->llap_checksum, csum);
+                        } else {
+                                /* mmapped page was modified */
+                                CDEBUG(D_PAGE,
+                                       "page %p old cksum %x != new %x\n",
+                                       page, llap->llap_checksum, csum);
+                        }
+                        llap->llap_checksum = csum;
                 }
         }
 
@@ -1497,23 +1515,26 @@ static int ll_read_ahead_page(struct obd_export *exp, struct obd_io_group *oig,
         if (Page_Uptodate(page))
                 GOTO(unlock_page, rc = 0);
 
+        if (!lustre_handle_is_used(&lockh)) {
+                start = ((loff_t)index) << CFS_PAGE_SHIFT;
+                end = start + CFS_PAGE_SIZE - 1;
+                rc = obd_get_lock(exp,
+                                  ll_i2info(mapping->host)->lli_smd,
+                                  &llap->llap_cookie, OBD_BRW_READ, 
+                                  start, end, &lockh, OBD_FAST_LOCK);
+                /* Is the lock being cancelling? */
+                if (rc <= 0)
+                        GOTO(unlock_page, rc = 0);
+        }
+
+        llap->llap_lockh_granted = lockh;
+
         rc = ll_issue_page_read(exp, llap, oig, 1);
         if (rc == 0) {
                 LL_CDEBUG_PAGE(D_READA | D_PAGE, page, "started read-ahead\n");
                 rc = 1;
-
-                if (!lustre_handle_is_used(&lockh)) {
-                        start = ((loff_t)index) << CFS_PAGE_SHIFT;
-                        end = start + CFS_PAGE_SIZE - 1;
-                        rc = obd_get_lock(exp,
-                                          ll_i2info(mapping->host)->lli_smd,
-                                          &llap->llap_cookie, OBD_BRW_READ, 
-                                          start, end, &lockh, OBD_FAST_LOCK);
-                        LASSERT(rc);
-                }
-
-                llap->llap_lockh_granted = lockh;
         } else {
+                llap->llap_lockh_granted.cookie = 0;
 unlock_page:
                 if (lustre_handle_is_used(&lockh))
                         ldlm_lock_decref(&lockh, LCK_PR);
