@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -79,7 +79,7 @@ void mdc_close_pack(struct ptlrpc_request *req, int offset,
                     struct obdo *oa,
                     __u64 valid, struct obd_client_handle *och);
 void mdc_exit_request(struct client_obd *cli);
-void mdc_enter_request(struct client_obd *cli);
+int mdc_enter_request(struct client_obd *cli);
 
 int seq_client_init(struct lu_client_seq *seq,
                     struct obd_export *exp,
@@ -91,6 +91,8 @@ void seq_client_fini(struct lu_client_seq *seq);
 
 int mdc_fid_alloc(struct lu_client_seq *seq, struct lu_fid *fid);
 
+void seq_client_flush(struct lu_client_seq *seq);
+
 struct mdc_open_data {
         struct obd_client_handle *mod_och;
         struct ptlrpc_request    *mod_open_req;
