@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2001, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -148,7 +148,9 @@ int mdc_getattr_common(struct obd_export *exp, unsigned int ea_size,
 
         ptlrpc_req_set_repsize(req, bufcount, size);
 
-        mdc_enter_request(&obddev->u.cli);
+        rc = mdc_enter_request(&obddev->u.cli);
+        if (rc != 0)
+                RETURN(rc);
         rc = ptlrpc_queue_wait(req);
         mdc_exit_request(&obddev->u.cli);
         if (rc != 0)
@@ -271,7 +273,7 @@ int mdc_xattr_common(struct obd_export *exp, struct ll_fid *fid,
                         [REQ_REC_OFF + 1] = 0, /* capa */
                         [REQ_REC_OFF + 2] = 0, /* name */
                         [REQ_REC_OFF + 3] = 0 };
-        int rc, xattr_namelen = 0, bufcnt = 2, offset = REQ_REC_OFF + 1;
+        int rc = 0, xattr_namelen = 0, bufcnt = 2, offset = REQ_REC_OFF + 1;
         void *tmp;
         ENTRY;
 
@@ -348,8 +350,11 @@ int mdc_xattr_common(struct obd_export *exp, struct ll_fid *fid,
         /* make rpc */
         if (opcode == MDS_SETXATTR || opcode == MDS_REINT)
                 mdc_get_rpc_lock(exp->exp_obd->u.cli.cl_rpc_lock, NULL);
-        else
-                mdc_enter_request(&obddev->u.cli);
+        else {
+                rc = mdc_enter_request(&obddev->u.cli);
+                if (rc != 0)
+                        GOTO(err_out, rc);
+        }
 
         rc = ptlrpc_queue_wait(req);
 
@@ -1331,6 +1336,13 @@ static int mdc_import_event(struct obd_device *obd, struct obd_import *imp,
                 break;
         }
         case IMP_EVENT_INACTIVE: {
+                struct client_obd *cli = &obd->u.cli;
+                /* Flush current sequence to make client obtain new one
+                 * from server in case of disconnect/reconnect.
+                 * If range is already empty then no need to flush it. */
+                if (cli->cl_seq != NULL)
+                        seq_client_flush(cli->cl_seq);
+
                 rc = obd_notify_observer(obd, obd, OBD_NOTIFY_INACTIVE, NULL);
                 break;
         }
@@ -1691,6 +1703,7 @@ MODULE_LICENSE("GPL");
 EXPORT_SYMBOL(mdc_req2lustre_md);
 EXPORT_SYMBOL(mdc_free_lustre_md);
 EXPORT_SYMBOL(mdc_change_cbdata);
+EXPORT_SYMBOL(mdc_find_cbdata);
 EXPORT_SYMBOL(mdc_getstatus);
 EXPORT_SYMBOL(mdc_getattr);
 EXPORT_SYMBOL(mdc_getattr_name);
