@@ -520,8 +520,12 @@ void mds_stats_counter_init(struct lprocfs_stats *stats)
         lprocfs_counter_init(stats, LPROC_MDS_MKDIR, 0, "mkdir", "reqs");
         lprocfs_counter_init(stats, LPROC_MDS_RMDIR, 0, "rmdir", "reqs");
         lprocfs_counter_init(stats, LPROC_MDS_RENAME, 0, "rename", "reqs");
+        lprocfs_counter_init(stats, LPROC_MDS_GETATTR, 0, "getattr", "reqs");
+        lprocfs_counter_init(stats, LPROC_MDS_SETATTR, 0, "setattr", "reqs");
         lprocfs_counter_init(stats, LPROC_MDS_GETXATTR, 0, "getxattr", "reqs");
         lprocfs_counter_init(stats, LPROC_MDS_SETXATTR, 0, "setxattr", "reqs");
+        lprocfs_counter_init(stats, LPROC_MDS_STATFS, 0, "statfs", "reqs");
+        lprocfs_counter_init(stats, LPROC_MDS_SYNC, 0, "sync", "reqs");
 }
 
 void lprocfs_mds_init_vars(struct lprocfs_static_vars *lvars)
