@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -283,6 +283,10 @@ int mds_client_free(struct obd_export *exp)
         if (!med->med_lcd)
                 RETURN(0);
 
+        /* Do not erase record for recoverable client. */
+        if (obd->obd_fail && !exp->exp_failed)
+                GOTO(free, 0);
+
         /* XXX if lcd_uuid were a real obd_uuid, I could use obd_uuid_equals */
         if (!strcmp(med->med_lcd->lcd_uuid, obd->obd_uuid.uuid))
                 GOTO(free, 0);
@@ -767,8 +771,8 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
 
         dput(dentry);
 
-        dentry = lookup_one_len("__iopen__", cfs_fs_pwd(current->fs),
-                                strlen("__iopen__"));
+        dentry = ll_lookup_one_len("__iopen__", cfs_fs_pwd(current->fs),
+                                   strlen("__iopen__"));
         if (IS_ERR(dentry)) {
                 rc = PTR_ERR(dentry);
                 CERROR("cannot lookup __iopen__ directory: rc = %d\n", rc);
@@ -973,7 +977,7 @@ int mds_obd_create(struct obd_export *exp, struct obdo *oa,
         push_ctxt(saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
 
         sprintf(fidname, "%u.%u", tmpname, current->pid);
-        dchild = lookup_one_len(fidname, mds->mds_objects_dir, strlen(fidname));
+        dchild = ll_lookup_one_len(fidname, mds->mds_objects_dir, strlen(fidname));
         if (IS_ERR(dchild)) {
                 CERROR("getting neg dentry for obj: %u\n", tmpname);
                 GOTO(out_pop, rc = PTR_ERR(dchild));
