@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -69,7 +69,7 @@ static inline void mds_export_evict(struct obd_export *exp)
 
 #ifdef __KERNEL__
 /* Open counts for files.  No longer atomic, must hold inode->i_sem */
-# define mds_inode_oatomic(inode)    ((inode)->i_cindex)
+#define mds_inode_oatomic(inode)    (*((long*)&((inode)->i_pipe)))
 
 #define MDS_UP_READ_ORPHAN_SEM(i)          UP_READ_I_ALLOC_SEM(i)
 #define MDS_DOWN_READ_ORPHAN_SEM(i)        DOWN_READ_I_ALLOC_SEM(i)
@@ -237,8 +237,7 @@ int mds_lov_start_synchronize(struct obd_device *obd,
 int mds_post_mds_lovconf(struct obd_device *obd);
 int mds_notify(struct obd_device *obd, struct obd_device *watched,
                enum obd_notify_event ev, void *data);
-int mds_get_default_md(struct obd_device *obd, struct lov_mds_md *lmm,
-                       int *lmmsize);
+int mds_get_default_md(struct obd_device *obd, struct lov_user_md *lmm);
 int mds_convert_lov_ea(struct obd_device *obd, struct inode *inode,
                        struct lov_mds_md *lmm, int lmm_size,
                        __u64 connect_flags);
@@ -313,8 +312,12 @@ enum {
         LPROC_MDS_MKDIR,
         LPROC_MDS_RMDIR,
         LPROC_MDS_RENAME,
+        LPROC_MDS_GETATTR,
+        LPROC_MDS_SETATTR,
         LPROC_MDS_GETXATTR,
         LPROC_MDS_SETXATTR,
+        LPROC_MDS_STATFS,
+        LPROC_MDS_SYNC,
         LPROC_MDS_LAST,
 };
 void mds_counter_incr(struct obd_export *exp, int opcode);
