@@ -61,7 +61,7 @@
 #include <lustre_lib.h>
 #include "mds_internal.h"
 
-void mds_pack_inode2fid(struct ll_fid *fid, struct inode *inode)
+static void mds_pack_inode2fid(struct ll_fid *fid, struct inode *inode)
 {
         fid->id = inode->i_ino;
         fid->generation = inode->i_generation;
@@ -80,6 +80,7 @@ void mds_pack_inode2body(struct mds_body *b, struct inode *inode)
                 b->valid |= OBD_MD_FLSIZE | OBD_MD_FLBLOCKS | OBD_MD_FLATIME |
                             OBD_MD_FLMTIME | OBD_MD_FLRDEV;
 
+        mds_pack_inode2fid(&b->fid1, inode);
         b->ino = inode->i_ino;
         b->atime = LTIME_S(inode->i_atime);
         b->mtime = LTIME_S(inode->i_mtime);
@@ -89,9 +90,7 @@ void mds_pack_inode2body(struct mds_body *b, struct inode *inode)
         b->blocks = inode->i_blocks;
         b->uid = inode->i_uid;
         b->gid = inode->i_gid;
-        b->flags = (b->flags & MDS_BFLAG_EXT_FLAGS) |
-                    ll_inode_to_ext_flags(inode->i_flags,
-                                          !(b->flags & MDS_BFLAG_EXT_FLAGS));
+        b->flags = ll_inode_to_ext_flags(inode->i_flags, MDS_BFLAG_EXT_FLAGS);
         b->rdev = inode->i_rdev;
         /* Return the correct link count for orphan inodes */
         b->nlink = mds_inode_is_orphan(inode) ? 0 : inode->i_nlink;
@@ -235,7 +234,7 @@ static int mds_create_unpack(struct ptlrpc_request *req, int offset,
         if (lustre_msg_buflen(req->rq_reqmsg, offset + 3)) {
                 r->ur_dlm = lustre_swab_reqbuf(req, offset + 3,
                                                sizeof(*r->ur_dlm),
-                                               lustre_swab_ldlm_request); 
+                                               lustre_swab_ldlm_request);
                 if (r->ur_dlm == NULL)
                         RETURN (-EFAULT);
         }
@@ -303,7 +302,6 @@ static int mds_unlink_unpack(struct ptlrpc_request *req, int offset,
         if (r->ur_name == NULL)
                 RETURN(-EFAULT);
         r->ur_namelen = lustre_msg_buflen(req->rq_reqmsg, offset + 1);
-        
         if (lustre_msg_buflen(req->rq_reqmsg, offset + 2)) {
                 r->ur_dlm = lustre_swab_reqbuf(req, offset + 2,
                                                sizeof(*r->ur_dlm),
