@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -52,7 +52,9 @@
 #include <lustre_dlm.h>
 #include <lprocfs_status.h>
 
-#define ECHO_INIT_OBJID      0x1000000000000000ULL
+/* The echo objid needs to be below 2^32, because regular FID numbers are
+ * limited to 2^32 objects in f_oid for the FID_SEQ_ECHO range. b=23335 */
+#define ECHO_INIT_OID        0x10000000ULL
 #define ECHO_HANDLE_MAGIC    0xabcd0123fedc9876ULL
 
 #define ECHO_PERSISTENT_PAGES (ECHO_PERSISTENT_SIZE >> CFS_PAGE_SHIFT)
@@ -150,7 +152,7 @@ int echo_destroy(struct obd_export *exp, struct obdo *oa,
                 RETURN(-EINVAL);
         }
 
-        if (oa->o_id > obd->u.echo.eo_lastino || oa->o_id < ECHO_INIT_OBJID) {
+        if (oa->o_id > obd->u.echo.eo_lastino || oa->o_id < ECHO_INIT_OID) {
                 CERROR("bad destroy objid: "LPX64"\n", oa->o_id);
                 RETURN(-EINVAL);
         }
@@ -418,6 +420,7 @@ int echo_commitrw(int cmd, struct obd_export *export, struct obdo *oa,
 {
         struct obd_device *obd;
         struct niobuf_local *r = res;
+        struct niobuf_remote *rbc = rb;
         int i, vrc = 0;
         ENTRY;
 
@@ -450,33 +453,36 @@ int echo_commitrw(int cmd, struct obd_export *export, struct obdo *oa,
                               (oa->o_flags & OBD_FL_DEBUG_CHECK) != 0);
                 int j;
 
-                for (j = 0 ; j < obj->ioo_bufcnt ; j++, r++) {
-                        cfs_page_t *page = r->page;
-                        void *addr;
+                for (j = 0 ; j < obj->ioo_bufcnt ; j++, rbc++) {
+                        long l;
+                        for (l = rbc->len; l>0; l-=r->len, r++) {
+                                cfs_page_t *page = r->page;
+                                void *addr;
 
-                        if (page == NULL) {
-                                CERROR("null page objid "LPU64":%p, buf %d/%d\n",
-                                       obj->ioo_id, page, j, obj->ioo_bufcnt);
-                                GOTO(commitrw_cleanup, rc = -EFAULT);
+                                if (page == NULL) {
+                                        CERROR("null page objid "LPU64":%p, buf %d/%d\n",
+                                        obj->ioo_id, page, j, obj->ioo_bufcnt);
+                                        GOTO(commitrw_cleanup, rc = -EFAULT);
+                                }
+
+                                addr = cfs_kmap(page);
+
+                                CDEBUG(D_PAGE, "$$$$ use page %p, addr %p@"LPU64"\n",
+                                r->page, addr, r->offset);
+
+                                if (verify) {
+                                        vrc = echo_page_debug_check(page, obj->ioo_id,
+                                                                r->offset, r->len);
+                                        /* check all the pages always */
+                                        if (vrc != 0 && rc == 0)
+                                                rc = vrc;
+                                }
+
+                                cfs_kunmap(page);
+                                /* NB see comment above regarding persistent pages */
+                                OBD_PAGE_FREE(page);
+                                atomic_dec(&obd->u.echo.eo_prep);
                         }
-
-                        addr = cfs_kmap(page);
-
-                        CDEBUG(D_PAGE, "$$$$ use page %p, addr %p@"LPU64"\n",
-                               r->page, addr, r->offset);
-
-                        if (verify) {
-                                vrc = echo_page_debug_check(page, obj->ioo_id,
-                                                            r->offset, r->len);
-                                /* check all the pages always */
-                                if (vrc != 0 && rc == 0)
-                                        rc = vrc;
-                        }
-
-                        cfs_kunmap(page);
-                        /* NB see comment above regarding persistent pages */
-                        OBD_PAGE_FREE(page);
-                        atomic_dec(&obd->u.echo.eo_prep);
                 }
         }
         CDEBUG(D_PAGE, "%d pages remain after commit\n",
@@ -506,7 +512,7 @@ static int echo_setup(struct obd_device *obd, obd_count len, void *buf)
         ENTRY;
 
         spin_lock_init(&obd->u.echo.eo_lock);
-        obd->u.echo.eo_lastino = ECHO_INIT_OBJID;
+        obd->u.echo.eo_lastino = ECHO_INIT_OID;
 
         sprintf(ns_name, "echotgt-%s", obd->obd_uuid.uuid);
         obd->obd_namespace = ldlm_namespace_new(obd, ns_name,
