@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -528,6 +528,7 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
         int                     rc;
         int                     verify;
         int                     gfp_mask;
+        int                     brw_flags = 0;
 
         verify = ((oa->o_id) != ECHO_PERSISTENT_OBJID &&
                   (oa->o_valid & OBD_MD_FLFLAGS) != 0 &&
@@ -543,6 +544,9 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
             (count & (~CFS_PAGE_MASK)) != 0)
                 return (-EINVAL);
 
+        if (rw == OBD_BRW_WRITE)
+                brw_flags = OBD_BRW_ASYNC;
+
         set =  ptlrpc_prep_set();
         if (set == NULL)
                 RETURN(-ENOMEM);
@@ -551,8 +555,10 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
         npages = count >> CFS_PAGE_SHIFT;
 
         OBD_ALLOC(pga, npages * sizeof(*pga));
-        if (pga == NULL)
-                return (-ENOMEM);
+        if (pga == NULL) {
+                rc = -ENOMEM;
+                goto out_set;
+        }
 
         for (i = 0, pgp = pga, off = offset;
              i < npages;
@@ -567,7 +573,7 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
 
                 pgp->count = CFS_PAGE_SIZE;
                 pgp->off = off;
-                pgp->flag = 0;
+                pgp->flag = brw_flags;
 
                 if (verify)
                         echo_client_page_debug_setup(lsm, pgp->pg, rw,
@@ -593,7 +599,6 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
         if (rc)
                 CDEBUG_LIMIT(rc == -ENOSPC ? D_INODE : D_ERROR,
                              "error from obd_brw_async: rc = %d\n", rc);
-        ptlrpc_set_destroy(set);
  out:
         if (rc != 0 || rw != OBD_BRW_READ)
                 verify = 0;
@@ -612,6 +617,8 @@ static int echo_client_kbrw(struct obd_device *obd, int rw, struct obdo *oa,
                 OBD_PAGE_FREE(pgp->pg);
         }
         OBD_FREE(pga, npages * sizeof(*pga));
+ out_set:
+        ptlrpc_set_destroy(set);
         return (rc);
 }
 
@@ -715,6 +722,7 @@ static int echo_client_async_page(struct obd_export *exp, int rw,
         struct echo_async_state *eas;
         int rc = 0;
         struct echo_async_page **aps = NULL;
+        int brw_flags = 0;
 
         ENTRY;
 #if 0
@@ -743,6 +751,9 @@ static int echo_client_async_page(struct obd_export *exp, int rw,
         if (NULL == eas)
                 return(-ENOMEM);
 
+        if (rw == OBD_BRW_WRITE)
+                brw_flags = OBD_BRW_ASYNC;
+
         memcpy(&eas->eas_oa, oa, sizeof(*oa));
         eas->eas_next_offset = offset;
         eas->eas_end_offset = offset + count;
@@ -828,7 +839,7 @@ static int echo_client_async_page(struct obd_export *exp, int rw,
 
                 /* always asserts urgent, which isn't quite right */
                 rc = obd_queue_async_io(exp, lsm, NULL, eap->eap_cookie,
-                                        rw, 0, CFS_PAGE_SIZE, 0,
+                                        rw, 0, CFS_PAGE_SIZE, brw_flags,
                                         ASYNC_READY | ASYNC_URGENT |
                                         ASYNC_COUNT_STABLE);
                 spin_lock(&eas->eas_lock);
@@ -855,15 +866,17 @@ static int echo_client_async_page(struct obd_export *exp, int rw,
 out:
         if (aps != NULL) {
                 for (i = 0; i < npages; ++ i) {
-                        cfs_page_t *page;
-
                         eap = aps[i];
-                        page = eap->eap_page;
-                        if (eap->eap_cookie != NULL)
-                                obd_teardown_async_page(exp, lsm, NULL,
-                                                        eap->eap_cookie);
-                        OBD_FREE(eap, sizeof(*eap));
-                        OBD_PAGE_FREE(page);
+                        if (eap != NULL) {
+                                cfs_page_t *page;
+
+                                page = eap->eap_page;
+                                if (eap->eap_cookie != NULL)
+                                        obd_teardown_async_page(exp, lsm, NULL,
+                                                                eap->eap_cookie);
+                                OBD_FREE(eap, sizeof(*eap));
+                                OBD_PAGE_FREE(page);
+                        }
                 }
                 OBD_FREE(aps, npages * sizeof aps[0]);
         }
@@ -1398,7 +1411,8 @@ static int echo_client_cleanup(struct obd_device *obddev)
                 el = ec->ec_objects.next;
                 eco = list_entry(el, struct ec_object, eco_obj_chain);
 
-                LASSERT(eco->eco_refcount == 0);
+                if (eco->eco_refcount > 0)
+                        RETURN(-EBUSY);
                 eco->eco_refcount = 1;
                 eco->eco_deleted = 1;
                 echo_put_object(eco);
