@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -255,6 +255,19 @@ static int osc_wr_cur_grant_bytes(struct file *file, const char *buffer,
         return count;
 }
 
+static int osc_rd_cur_lost_grant_bytes(char *page, char **start, off_t off,
+                                       int count, int *eof, void *data)
+{
+        struct obd_device *dev = data;
+        struct client_obd *cli = &dev->u.cli;
+        int rc;
+
+        client_obd_list_lock(&cli->cl_loi_list_lock);
+        rc = snprintf(page, count, "%lu\n", cli->cl_lost_grant);
+        client_obd_list_unlock(&cli->cl_loi_list_lock);
+        return rc;
+}
+
 static int osc_rd_grant_shrink_interval(char *page, char **start, off_t off,
                                         int count, int *eof, void *data)
 {
@@ -508,6 +521,14 @@ static int osc_wr_resend_count(struct file *file, const char *buffer,
         return count;
 }
 
+static int osc_rd_destroys_in_flight(char *page, char **start, off_t off,
+                                     int count, int *eof, void *data)
+{
+        struct obd_device *obd = data;
+        return snprintf(page, count, "%u\n",
+                        atomic_read(&obd->u.cli.cl_destroy_in_flight));
+}
+
 static struct lprocfs_vars lprocfs_osc_obd_vars[] = {
         { "uuid",            lprocfs_rd_uuid,        0, 0 },
         { "ping",            0, lprocfs_wr_ping,     0, 0, 0222 },
@@ -527,10 +548,12 @@ static struct lprocfs_vars lprocfs_osc_obd_vars[] = {
                                osc_wr_max_pages_per_rpc, 0 },
         { "max_rpcs_in_flight", osc_rd_max_rpcs_in_flight,
                                 osc_wr_max_rpcs_in_flight, 0 },
+        { "destroys_in_flight", osc_rd_destroys_in_flight, 0, 0 },
         { "max_dirty_mb",    osc_rd_max_dirty_mb, osc_wr_max_dirty_mb, 0 },
         { "cur_dirty_bytes", osc_rd_cur_dirty_bytes, 0, 0 },
         { "cur_grant_bytes", osc_rd_cur_grant_bytes,
                              osc_wr_cur_grant_bytes, 0 },
+        { "cur_lost_grant_bytes", osc_rd_cur_lost_grant_bytes, 0, 0},
         { "grant_shrink_interval", osc_rd_grant_shrink_interval,
                                    osc_wr_grant_shrink_interval, 0 },
         { "create_count",    osc_rd_create_count, osc_wr_create_count, 0 },
@@ -566,15 +589,19 @@ static int osc_rpc_stats_seq_show(struct seq_file *seq, void *v)
 
         client_obd_list_lock(&cli->cl_loi_list_lock);
 
-        seq_printf(seq, "snapshot_time:         %lu.%lu (secs.usecs)\n",
+        seq_printf(seq, "snapshot_time:            %lu.%lu (secs.usecs)\n",
                    now.tv_sec, now.tv_usec);
-        seq_printf(seq, "read RPCs in flight:  %d\n",
+        seq_printf(seq, "read RPCs in flight:      %d\n",
                    cli->cl_r_in_flight);
-        seq_printf(seq, "write RPCs in flight: %d\n",
+        seq_printf(seq, "write RPCs in flight:     %d\n",
                    cli->cl_w_in_flight);
-        seq_printf(seq, "pending write pages:  %d\n",
+        seq_printf(seq, "dio read RPCs in flight:  %d\n",
+                   cli->cl_dio_r_in_flight);
+        seq_printf(seq, "dio write RPCs in flight: %d\n",
+                   cli->cl_dio_w_in_flight);
+        seq_printf(seq, "pending write pages:      %d\n",
                    cli->cl_pending_w_pages);
-        seq_printf(seq, "pending read pages:   %d\n",
+        seq_printf(seq, "pending read pages:       %d\n",
                    cli->cl_pending_r_pages);
 
         seq_printf(seq, "\n\t\t\tread\t\t\twrite\n");
