@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -62,8 +62,7 @@ void request_out_callback(lnet_event_t *ev)
                  ev->type == LNET_EVENT_UNLINK);
         LASSERT (ev->unlinked);
 
-        DEBUG_REQ((ev->status == 0) ? D_NET : D_ERROR, req,
-                  "type %d, status %d", ev->type, ev->status);
+        DEBUG_REQ(D_NET, req, "type %d, status %d", ev->type, ev->status);
 
         if (ev->type == LNET_EVENT_UNLINK || ev->status != 0) {
 
@@ -91,8 +90,7 @@ void reply_in_callback(lnet_event_t *ev)
         struct ptlrpc_request *req = cbid->cbid_arg;
         ENTRY;
 
-        DEBUG_REQ((ev->status == 0) ? D_NET : D_ERROR, req,
-                  "type %d, status %d", ev->type, ev->status);
+        DEBUG_REQ(D_NET, req, "type %d, status %d", ev->type, ev->status);
 
         LASSERT(ev->type == LNET_EVENT_PUT || ev->type == LNET_EVENT_UNLINK);
         LASSERT(ev->md.start == req->rq_repbuf);
@@ -113,7 +111,7 @@ void reply_in_callback(lnet_event_t *ev)
 
         if (ev->type == LNET_EVENT_UNLINK) {
                 LASSERT(ev->unlinked);
-                DEBUG_REQ(D_RPCTRACE, req, "unlink");
+                DEBUG_REQ(D_NET, req, "unlink");
                 goto out_wake;
         }
 
@@ -287,7 +285,7 @@ void request_in_callback(lnet_event_t *ev)
         CFS_INIT_LIST_HEAD(&req->rq_exp_list);
         atomic_set(&req->rq_refcount, 1);
         if (ev->type == LNET_EVENT_PUT)
-                DEBUG_REQ(D_RPCTRACE, req, "incoming req");
+                DEBUG_REQ(D_NET, req, "incoming req");
 
         spin_lock(&service->srv_lock);
 
@@ -472,8 +470,6 @@ int ptlrpc_uuid_to_peer (struct obd_uuid *uuid,
         }
 
         CDEBUG(D_NET,"%s->%s\n", uuid->uuid, libcfs_id2str(*peer));
-        if (rc != 0) 
-                CERROR("No NID found for %s\n", uuid->uuid);
         return rc;
 }
 
