@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -1429,6 +1429,13 @@ void ptlrpc_import_setasync(struct obd_import *imp, int count)
         LNetSetAsync(imp->imp_connection->c_peer, count);
 }
 
+void ptlrpc_evict_imp(struct obd_import *imp)
+{
+        ENTRY;
+        IMPORT_SET_STATE(imp, LUSTRE_IMP_EVICTED);
+        EXIT;
+}
+
 void ptlrpc_cleanup_imp(struct obd_import *imp)
 {
         ENTRY;
