@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -507,7 +507,10 @@ int ptl_send_rpc(struct ptlrpc_request *request, int noreply)
                 CDEBUG(D_HA, "muting rpc for failed imp obd %s\n",
                        request->rq_import->imp_obd->obd_name);
                 /* this prevents us from waiting in ptlrpc_queue_wait */
+                spin_lock(&request->rq_lock);
                 request->rq_err = 1;
+                spin_unlock(&request->rq_lock);
+
                 request->rq_status = -ENODEV;
                 RETURN(-ENODEV);
         }
@@ -537,7 +540,10 @@ int ptl_send_rpc(struct ptlrpc_request *request, int noreply)
                         OBD_ALLOC(request->rq_repbuf, request->rq_replen);
                 if (request->rq_repbuf == NULL) {
                         /* this prevents us from looping in ptlrpc_queue_wait */
+                        spin_lock(&request->rq_lock);
                         request->rq_err = 1;
+                        spin_unlock(&request->rq_lock);
+
                         request->rq_status = -ENOMEM;
                         GOTO(cleanup_bulk, rc = -ENOMEM);
                 }
