@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -107,10 +107,10 @@ int ptlrpcd_add_req(struct ptlrpc_request *req)
 
                 req->rq_invalid_rqset = 0;
                 spin_unlock(&req->rq_lock);
-                
+
                 timeout = cfs_time_seconds(5);
                 lwi = LWI_TIMEOUT(timeout, back_to_sleep, NULL);
-                l_wait_event(req->rq_reply_waitq, (req->rq_set == NULL), &lwi);
+                l_wait_event(req->rq_set_waitq, (req->rq_set == NULL), &lwi);
         } else if (req->rq_set) {
                 LASSERT(req->rq_phase == RQ_PHASE_NEW);
                 LASSERT(req->rq_send_state == LUSTRE_IMP_REPLAY);
