@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -61,6 +61,10 @@
 #include <lprocfs_status.h>
 #include "quota_internal.h"
 
+static int hash_lqs_cur_bits = HASH_LQS_CUR_BITS;
+CFS_MODULE_PARM(hash_lqs_cur_bits, "i", int, 0444,
+                "the current bits of lqs hash");
+
 #ifdef HAVE_QUOTA_SUPPORT
 
 static lustre_hash_ops_t lqs_hash_ops;
@@ -1256,9 +1260,9 @@ qctxt_init(struct obd_device *obd, dqacq_handler_t handler)
         spin_unlock(&qctxt->lqc_lock);
 
         qctxt->lqc_lqs_hash = lustre_hash_init("LQS_HASH",
-                                               HASH_LQS_CUR_BITS,
-                                               HASH_LQS_CUR_BITS,
-                                               &lqs_hash_ops, 0);
+                                               hash_lqs_cur_bits,
+                                               HASH_LQS_MAX_BITS,
+                                               &lqs_hash_ops, LH_REHASH);
         if (!qctxt->lqc_lqs_hash)
                 CERROR("%s: initialize hash lqs failed\n", obd->obd_name);
 
@@ -1485,18 +1489,10 @@ lqs_key(struct hlist_node *hnode)
 static int
 lqs_compare(void *key, struct hlist_node *hnode)
 {
-        struct lustre_qunit_size *q;
-        int rc;
-        ENTRY;
+        struct lustre_qunit_size *q =
+            hlist_entry(hnode, struct lustre_qunit_size, lqs_hash);
 
-        LASSERT(key);
-        q = hlist_entry(hnode, struct lustre_qunit_size, lqs_hash);
-
-        spin_lock(&q->lqs_lock);
-        rc = (q->lqs_key == *((unsigned long long *)key));
-        spin_unlock(&q->lqs_lock);
-
-        RETURN(rc);
+        RETURN(q->lqs_key == *((unsigned long long *)key));
 }
 
 static void *
