@@ -15,6 +15,12 @@ ONLY=${ONLY:-"$*"}
 ALWAYS_EXCEPT=" $CONF_SANITY_EXCEPT 34a"
 # UPDATE THE COMMENT ABOVE WITH BUG NUMBERS WHEN CHANGING ALWAYS_EXCEPT!
 
+if [ "$FAILURE_MODE" = "HARD" ]; then
+	CONFIG_EXCEPTIONS="24a " && \
+	echo "Except the tests: $CONFIG_EXCEPTIONS for FAILURE_MODE=$FAILURE_MODE, bug 23573" && \
+	ALWAYS_EXCEPT="$ALWAYS_EXCEPT $CONFIG_EXCEPTIONS"
+fi
+
 SRCDIR=`dirname $0`
 PATH=$PWD/$SRCDIR:$SRCDIR:$SRCDIR/../utils:$PATH
 
@@ -38,7 +44,7 @@ remote_mds_nodsh && skip "remote MDS with nodsh" && exit 0
 remote_ost_nodsh && skip "remote OST with nodsh" && exit 0
 
 #
-[ "$SLOW" = "no" ] && EXCEPT_SLOW="0 1 2 3 6 7 15 18 24b 25 30 31 32 33 34a 45"
+[ "$SLOW" = "no" ] && EXCEPT_SLOW="30 31 45"
 
 assert_DIR
 
@@ -46,15 +52,25 @@ reformat() {
         formatall
 }
 
-writeconf() {
-	local facet=mds
-	shift
+writeconf1() {
+	local facet=$1
+	local dev=$2
+
 	stop ${facet} -f
 	rm -f ${facet}active
 	# who knows if/where $TUNEFS is installed?  Better reformat if it fails...
-	do_facet ${facet} "$TUNEFS --writeconf $MDSDEV" ||
-		echo "tunefs failed, reformatting instead" && reformat_and_config
+	do_facet ${facet} "$TUNEFS --quiet --writeconf $dev" ||
+		{ echo "tunefs failed, reformatting instead" && reformat_and_config && return 1; }
+	return 0
+}
 
+writeconf() {
+	# we need ldiskfs
+	load_modules
+	# if writeconf fails anywhere, we reformat everything
+	writeconf1 mds $MDSDEV || return 0
+	writeconf1 ost1 `ostdevname 1` || return 0
+	writeconf1 ost2 `ostdevname 2` || return 0
 }
 
 gen_config() {
@@ -77,12 +93,25 @@ reformat_and_config() {
 
 start_mgs () {
 	echo "start mgs"
-	start mgs $MGSDEV $mgs_MOUNT_OPTS
+	start mgs $MGSDEV $MGS_MOUNT_OPTS
 }
 
 start_mds() {
 	echo "start mds service on `facet_active_host mds`"
-	start mds $MDSDEV $MDS_MOUNT_OPTS || return 94
+	start mds $MDSDEV $MDS_MOUNT_OPTS $@ || return 94
+}
+
+stop_mgs() {
+	echo "stop mgs service on `facet_active_host mgs`"
+	# These tests all use non-failover stop
+	stop mgs -f  || return 97
+}
+
+start_mgsmds() {
+	if ! combined_mgs_mds ; then
+		start_mgs
+	fi
+	start_mds $@
 }
 
 stop_mds() {
@@ -93,7 +122,7 @@ stop_mds() {
 
 start_ost() {
 	echo "start ost1 service on `facet_active_host ost1`"
-	start ost1 `ostdevname 1` $OST_MOUNT_OPTS || return 95
+	start ost1 `ostdevname 1` $OST_MOUNT_OPTS $@ || return 95
 }
 
 stop_ost() {
@@ -104,7 +133,7 @@ stop_ost() {
 
 start_ost2() {
 	echo "start ost2 service on `facet_active_host ost2`"
-	start ost2 `ostdevname 2` $OST_MOUNT_OPTS || return 92
+	start ost2 `ostdevname 2` $OST_MOUNT_OPTS $@ || return 92
 }
 
 stop_ost2() {
@@ -154,9 +183,9 @@ manual_umount_client(){
 }
 
 setup() {
-	start_ost
-	start_mds
-	mount_client $MOUNT
+	start_mds || error "MDT start failed"
+	start_ost || error "OST start failed"
+	mount_client $MOUNT || error "client start failed"
 	df $MOUNT
 }
 
@@ -224,7 +253,9 @@ run_test 0 "single mount setup"
 test_1() {
 	start_ost
 	echo "start ost second time..."
-	setup
+	start_ost && error "2nd OST start should fail"
+	start_mds || error "MDT start failed"
+	mount_client $MOUNT || error "client start failed"
 	check_mount || return 42
 	cleanup || return $?
 }
@@ -234,7 +265,7 @@ test_2() {
 	start_ost
 	start_mds
 	echo "start mds second time.."
-	start_mds
+	start_mds && error "2nd MDT start should fail"
 	mount_client $MOUNT
 	check_mount || return 43
 	cleanup || return $?
@@ -244,7 +275,7 @@ run_test 2 "start up mds twice (should return err)"
 test_3() {
 	setup
 	#mount.lustre returns an error if already in mtab
-	mount_client $MOUNT && return $?
+	mount_client $MOUNT && error "2nd client mount should fail"
 	check_mount || return 44
 	cleanup || return $?
 }
@@ -313,60 +344,132 @@ test_5a() {	# was test_5
 }
 run_test 5a "force cleanup mds, then cleanup"
 
+cleanup_5b () {
+	trap 0
+	start_mgs
+}
+
 test_5b() {
+	grep " $MOUNT " /etc/mtab && \
+		error false "unexpected entry in mtab before mount" && return 10
+
+	local rc=0
 	start_ost
+	if ! combined_mgs_mds ; then
+		trap cleanup_5b EXIT ERR
+		start_mds
+		stop mgs
+	fi
+
 	[ -d $MOUNT ] || mkdir -p $MOUNT
-	grep " $MOUNT " /etc/mtab && echo "test 5b: mtab before mount" && return 10
-	mount_client $MOUNT && return 1
-	grep " $MOUNT " /etc/mtab && echo "test 5b: mtab after failed mount" && return 11
+	mount_client $MOUNT && rc=1
+	grep " $MOUNT " /etc/mtab && \
+		error "$MOUNT entry in mtab after failed mount" && rc=11
 	umount_client $MOUNT
 	# stop_mds is a no-op here, and should not fail
-	cleanup_nocli || return $?
-	return 0
+	cleanup_nocli || rc=$?
+	if ! combined_mgs_mds ; then
+		cleanup_5b
+	fi
+	return $rc
 }
-run_test 5b "mds down, cleanup after failed mount (bug 2712) (should return errs)"
+run_test 5b "Try to start a client with no MGS (should return errs)"
 
 test_5c() {
+	grep " $MOUNT " /etc/mtab && \
+		error false "unexpected entry in mtab before mount" && return 10
+
+	local rc=0
 	start_ost
 	start_mds
 	[ -d $MOUNT ] || mkdir -p $MOUNT
-	grep " $MOUNT " /etc/mtab && echo "test 5c: mtab before mount" && return 10
 	local oldfs="${FSNAME}"
 	FSNAME="wrong.${FSNAME}"
 	mount_client $MOUNT || :
 	FSNAME=${oldfs}
-	grep " $MOUNT " /etc/mtab && echo "test 5c: mtab after failed mount" && return 11
+	grep " $MOUNT " /etc/mtab && \
+		error "$MOUNT entry in mtab after failed mount" && rc=11
 	umount_client $MOUNT
-	cleanup_nocli  || return $?
+	cleanup_nocli  || rc=$?
+	return $rc
 }
 run_test 5c "cleanup after failed mount (bug 2712) (should return errs)"
 
 test_5d() {
+	grep " $MOUNT " /etc/mtab && \
+		error false "unexpected entry in mtab before mount" && return 10
+
+	local rc=0
 	start_ost
 	start_mds
 	stop_ost -f
-	grep " $MOUNT " /etc/mtab && echo "test 5d: mtab before mount" && return 10
-	mount_client $MOUNT || return 1
-	cleanup  || return $?
-	grep " $MOUNT " /etc/mtab && echo "test 5d: mtab after unmount" && return 11
-	return 0
+	mount_client $MOUNT || rc=1
+	cleanup  || rc=$?
+	grep " $MOUNT " /etc/mtab && \
+		error "$MOUNT entry in mtab after unmount" && rc=11
+	return $rc
 }
 run_test 5d "mount with ost down"
 
 test_5e() {
+	grep " $MOUNT " /etc/mtab && \
+		error false "unexpected entry in mtab before mount" && return 10
+
+	local rc=0
 	start_ost
 	start_mds
 
 #define OBD_FAIL_PTLRPC_DELAY_SEND       0x506
 	do_facet client "lctl set_param fail_loc=0x80000506"
-	grep " $MOUNT " /etc/mtab && echo "test 5e: mtab before mount" && return 10
 	mount_client $MOUNT || echo "mount failed (not fatal)"
-	cleanup  || return $?
-	grep " $MOUNT " /etc/mtab && echo "test 5e: mtab after unmount" && return 11
-	return 0
+	cleanup  || rc=$?
+	grep " $MOUNT " /etc/mtab && \
+		error "$MOUNT entry in mtab after unmount" && rc=11
+	return $rc
 }
 run_test 5e "delayed connect, don't crash (bug 10268)"
 
+test_5f() {
+	if combined_mgs_mds ; then
+		skip "combined mgs and mds"
+		return 0
+	fi
+
+	grep " $MOUNT " /etc/mtab && \
+		error false "unexpected entry in mtab before mount" && return 10
+
+	local rc=0
+	start_ost
+	[ -d $MOUNT ] || mkdir -p $MOUNT
+	mount_client $MOUNT &
+	local pid=$!
+	echo client_mount pid is $pid
+
+	sleep 5
+
+	if ! ps -f -p $pid >/dev/null; then 
+		wait $pid
+		rc=$?
+		grep " $MOUNT " /etc/mtab && echo "test 5f: mtab after mount"
+		error "mount returns $rc, expected to hang"
+		rc=11
+		cleanup || rc=$?
+		return $rc
+	fi
+
+	# start mds
+	start_mds
+
+	# mount should succeed after start mds 
+	wait $pid
+	rc=$?
+	[ $rc -eq 0 ] || error "mount returned $rc"
+	grep " $MOUNT " /etc/mtab && echo "test 5f: mtab after mount"
+	cleanup || return $?
+	return $rc
+}
+run_test 5f "mds down, cleanup after failed mount (bug 2712)"
+
 test_6() {
 	setup
 	manual_umount_client
@@ -420,7 +523,6 @@ test_9() {
         fi
         stop_ost || return $?
 }
-
 run_test 9 "test ptldebug and subsystem for mkfs"
 
 is_blkdev () {
@@ -440,17 +542,18 @@ is_blkdev () {
 
 test_16() {
         setup
-        # interop 1.8 <-> 2.0: 20: no LOGS
-        local files="{OBJECTS,PENDING}"
-        local lustre_version=$(get_lustre_version mds)
-        if [[ $lustre_version = 1.8* ]]; then
-                files="{OBJECTS,PENDING,LOGS}"
+        # Skip this test if MDS is 2.0+, see bug 22944.
+        if [[ "$(get_mds_version_major)" != "1" ]]; then
+                skip "not running on 2.0 server"
+                cleanup
+                return 0
         fi
-        
+
         check_mount || return 41
         cleanup || return $?
 
         local TMPMTPT="${TMP}/conf16"
+        local files="{OBJECTS,PENDING,LOGS}"
 
         is_blkdev mds $MDSDEV || LOOPOPT="-o loop"
 
@@ -630,6 +733,28 @@ test_21c() {
 }
 run_test 21c "start mds between two osts, stop mds last"
 
+test_21d() {
+	if combined_mgs_mds ; then
+		skip "need separate mgs device" && return 0
+	fi
+	stopall
+
+	reformat
+
+	start_mgs
+	start_ost
+	start_ost2
+	start_mds
+	wait_osc_import_state mds ost2 FULL
+
+	stop_ost
+	stop_ost2
+	stop_mds
+	stop_mgs
+}
+run_test 21d "start mgs then ost and then mds"
+
+
 test_22() {
 	start_mds
 
@@ -771,8 +896,11 @@ test_24a() {
  	umount_client $MOUNT
 	# the MDS must remain up until last MDT
 	stop_mds
-	MDS=$(do_facet mds "lctl get_param -n devices" | awk '($3 ~ "mdt" && $4 ~ "MDS") { print $4 }')
-	[ -z "$MDS" ] && error "No MDS" && return 8
+
+	local mdt_obdname=$(do_facet mds "lctl get_param -n devices" | \
+        awk '/ mdt / { print $4 }' | head -1)
+	[ -z "$mdt_obdname" ] && error "No MDT" && return 8
+	
 	cleanup_24a
 	cleanup_nocli || return 6
 }
@@ -947,9 +1075,10 @@ test_29() {
 }
 run_test 29 "permanently remove an OST"
 
-test_30() {
+test_30a() {
 	setup
 
+	echo Big config llog
 	TEST="lctl get_param -n llite.$FSNAME-*.max_read_ahead_whole_mb"
 	ORIG=$($TEST)
 	LIST=(1 2 3 4 5 4 3 2 1 2 3 4 5 4 3 2 1 2 3 4 5)
@@ -959,11 +1088,54 @@ test_30() {
 	# make sure client restart still works
  	umount_client $MOUNT
 	mount_client $MOUNT || return 4
-	[ "$($TEST)" -ne "$i" ] && return 5
-	set_and_check client "$TEST" "$FSNAME.llite.max_read_ahead_whole_mb" $ORIG || return 6
+	[ "$($TEST)" -ne "$i" ] && error "Param didn't stick across restart $($TEST) != $i"
+	pass
+
+	echo Erase parameter setting
+	do_facet mgs "$LCTL conf_param -d $FSNAME.llite.max_read_ahead_whole_mb" || return 6
+	umount_client $MOUNT
+	mount_client $MOUNT || return 6
+	FINAL=$($TEST)
+	echo "deleted (default) value=$FINAL, orig=$ORIG"
+	# assumes this parameter started at the default value
+	[ "$FINAL" -eq "$ORIG" ] || fail "Deleted value=$FINAL, orig=$ORIG"
+
 	cleanup
 }
-run_test 30 "Big config llog"
+run_test 30a "Big config llog and conf_param deletion"
+
+test_30b() {
+	setup
+
+	# Make a fake nid.  Use the OST nid, and add 20 to the least significant
+	# numerical part of it. Hopefully that's not already a failover address for
+	# the server.
+	OSTNID=$(do_facet ost1 "$LCTL get_param nis" | tail -1 | awk '{print $1}')
+	ORIGVAL=$(echo $OSTNID | egrep -oi "[0-9]*@")
+	NEWVAL=$((($(echo $ORIGVAL | egrep -oi "[0-9]*") + 20) % 256))
+	NEW=$(echo $OSTNID | sed "s/$ORIGVAL/$NEWVAL@/")
+	echo "Using fake nid $NEW"
+
+	TEST="$LCTL get_param -n osc.$FSNAME-OST0000-osc-[^M]*.import | grep failover_nids | sed -n 's/.*\($NEW\).*/\1/p'"
+	set_and_check client "$TEST" "$FSNAME-OST0000.failover.node" $NEW || error "didn't add failover nid $NEW"
+	NIDS=$($LCTL get_param -n osc.$FSNAME-OST0000-osc-[^M]*.import | grep failover_nids)
+	echo $NIDS
+	NIDCOUNT=$(($(echo "$NIDS" | wc -w) - 1))
+	echo "should have 2 failover nids: $NIDCOUNT"
+	[ $NIDCOUNT -eq 2 ] || error "Failover nid not added"
+	do_facet mgs "$LCTL conf_param -d $FSNAME-OST0000.failover.node" || error "conf_param delete failed"
+	umount_client $MOUNT
+	mount_client $MOUNT || return 3
+
+	NIDS=$($LCTL get_param -n osc.$FSNAME-OST0000-osc-[^M]*.import | grep failover_nids)
+	echo $NIDS
+	NIDCOUNT=$(($(echo "$NIDS" | wc -w) - 1))
+	echo "only 1 final nid should remain: $NIDCOUNT"
+	[ $NIDCOUNT -eq 1 ] || error "Failover nids not removed"
+
+	cleanup
+}
+run_test 30b "Remove failover nids"
 
 test_31() { # bug 10734
         # ipaddr must not exist
@@ -1114,9 +1286,11 @@ test_32b() {
 	echo OST uuid $UUID
 	[ "$UUID" == "lustre-OST0000_UUID" ] || error "UUID is wrong: $UUID"
 
+	local NID=$($LCTL list_nids | head -1)
+
 	echo "OSC changes should succeed:"
-	$LCTL conf_param lustre-OST0000.osc.max_dirty_mb=15 || return 7
-	$LCTL conf_param lustre-OST0000.failover.node=$NID || return 8
+	$LCTL conf_param lustre-OST0000.osc.max_dirty_mb=15 || error "OSC conf_param failed"
+	$LCTL conf_param lustre-OST0000.failover.node=$NID || error "add failover nid=$NID failed"
 	echo "ok."
 	echo "MDC changes should succeed:"
 	$LCTL conf_param lustre-MDT0000.mdc.max_rpcs_in_flight=9 || return 9
@@ -1154,9 +1328,19 @@ test_33a() { # bug 12333, was test_33
                 skip_env "mixed loopback and real device not working" && return
         fi
 
+        # interop 1.8 <-> 2.0
+        local mkfs_opts
+        local major=$(get_mds_version_major)
+        local minor=$(get_mds_version_minor)
+        if [ $major -le 1 -a $minor -le 8 ]; then
+                mkfs_opts=""
+        else
+                mkfs_opts="--mkfsoptions='-J size=8'" # bug 17931
+        fi
+
         local fs2mdsdev=${fs2mds_DEV:-${MDSDEV}_2}
         local fs2ostdev=${fs2ost_DEV:-$(ostdevname 1)_2}
-        add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME2} --reformat $fs2mdsdev || exit 10
+        add fs2mds $MDS_MKFS_OPTS --fsname=${FSNAME2} $mkfs_opts --reformat $fs2mdsdev || exit 10
         add fs2ost $OST_MKFS_OPTS --fsname=${FSNAME2} --index=8191 --mgsnode=$MGSNID --reformat $fs2ostdev || exit 10
 
         start fs2mds $fs2mdsdev $MDS_MOUNT_OPTS && trap cleanup_24a EXIT INT
@@ -1282,6 +1466,8 @@ test_35a() { # bug 12459
 		       }" $TMP/lustre-log-$TESTNAME.log`
 	[ "$NEXTCONN" != "0" ] && log "The client didn't try to reconnect to the last active server (tried ${NEXTCONN} instead)" && return 7
 	cleanup
+	# remove nid settings
+	writeconf
 }
 run_test 35a "Reconnect to the last active server first"
 
@@ -1357,6 +1543,8 @@ test_35b() { # bug 18674
 		return 6
 
         cleanup
+	# remove nid settings
+	writeconf
 }
 run_test 35b "Continue reconnection retries, if the active server is busy"
 
@@ -1654,10 +1842,9 @@ cleanup_46a() {
 		let count=count-1
 	done	
 	stop_mds || rc=$? 
-	# writeconf is needed after the test, otherwise,
-	# we might end up with extra OSTs
-	writeconf || rc=$?
 	cleanup_nocli || rc=$?
+	#writeconf to remove all ost2 traces for subsequent tests
+	writeconf
 	return $rc
 }
 
@@ -1990,6 +2177,7 @@ test_50f() {
 	umount_client $MOUNT || error "Unable to unmount client"
 	stop_ost || error "Unable to stop OST1"
 	stop_mds || error "Unable to stop MDS"
+	#writeconf to remove all ost2 traces for subsequent tests
 	writeconf
 }
 run_test 50f "normal statfs one server in down =========================="
@@ -1998,6 +2186,8 @@ test_50g() {
 	[ "$OSTCOUNT" -lt "2" ] && skip_env "$OSTCOUNT < 2, skipping" && return
 	setup
 	start_ost2 || error "Unable to start OST2"
+	wait_osc_import_state mds ost2 FULL
+	wait_osc_import_state client ost2 FULL
 
 	local PARAM="${FSNAME}-OST0001.osc.active"
 
@@ -2015,13 +2205,139 @@ test_50g() {
 	stop_ost2 || error "Unable to stop OST2"
 	stop_ost || error "Unable to stop OST1"
 	stop_mds || error "Unable to stop MDS"
+	#writeconf to remove all ost2 traces for subsequent tests
 	writeconf
 }
 run_test 50g "deactivated OST should not cause panic====================="
 
+lov_objid_size()
+{
+	local max_ost_index=$1
+	echo -n $(($max_ost_index * 8 + 8))
+}
+
+test_55() {
+	local saved_opts=$OST_MKFS_OPTS
+
+	for i in 0 1023 2048
+	do
+		OST_MKFS_OPTS="$saved_opts --index $i"
+		reformat
+
+		setup_noconfig
+		cp /etc/passwd $DIR/1
+		stopall
+
+		setup
+		cp /etc/passwd $DIR/2
+		sync
+
+		echo checking size of lov_objid for ost index $i
+		local file_size=`do_facet mds \
+		    "$DEBUGFS -R 'stat lov_objid' $MDSDEV 2>/dev/null" | \
+		    grep ^User | awk '{print $6}'`
+		if [ "$file_size" != $(lov_objid_size $i) ]; then
+			error "lov_objid size has to be $(lov_objid_size $i), \
+not $file_size"
+		else
+			echo ok, lov_objid size is correct: $file_size
+		fi
+		stopall
+	done
+
+	OST_MKFS_OPTS=$saved_opts
+	reformat
+}
+run_test 55 "check lov_objid size"
+
+test_56() {
+	add mds $MDS_MKFS_OPTS --mkfsoptions='\"-J size=16\"' --reformat $MDSDEV
+	add ost1 $OST_MKFS_OPTS --index=1000 --reformat `ostdevname 1`
+	add ost2 $OST_MKFS_OPTS --index=10000 --reformat `ostdevname 2`
+
+	start_mds
+	start_ost
+	start_ost2 || error "Unable to start second ost"
+	mount_client $MOUNT || error "Unable to mount client"
+	[ -n "$ENABLE_QUOTA" ] && { $LFS quotacheck -ug $MOUNT || error "quotacheck has failed" ; }
+
+	stopall
+	reformat
+}
+run_test 56 "check big indexes"
+
+test_57() { # bug 22656
+	local NID=$(do_facet ost1 "$LCTL get_param nis" | tail -1 | awk '{print $1}')
+	writeconf
+	do_facet ost1 "$TUNEFS --failnode=$NID `ostdevname 1`" || error "tunefs failed"
+	start_mgsmds
+	stop_mds
+	start_ost && error "OST registration from failnode should fail"
+	reformat
+}
+run_test 57 "initial registration from failnode should fail (should return errs)"
+
+test_58() { # bug 22658
+        [ "$FSTYPE" != "ldiskfs" ] && skip "not supported for $FSTYPE" && return
+	setup
+	mkdir -p $DIR/$tdir
+	createmany -o $DIR/$tdir/$tfile-%d 100
+	# make sure that OSTs do not cancel llog cookies before we unmount the MDS
+#define OBD_FAIL_OBD_LOG_CANCEL_NET      0x601
+	do_facet mds "lctl set_param fail_loc=0x601"
+	unlinkmany $DIR/$tdir/$tfile-%d 100
+	stop mds
+	local MNTDIR=$(facet_mntpt mds)
+	# remove all files from the OBJECTS dir
+	do_facet mds "mount -t ldiskfs $MDSDEV $MNTDIR"
+	do_facet mds "find $MNTDIR/OBJECTS -type f -delete"
+	do_facet mds "umount $MNTDIR"
+	# restart MDS with missing llog files
+	start_mds
+	do_facet mds "lctl set_param fail_loc=0"
+	reformat
+}
+run_test 58 "missing llog files must not prevent MDT from mounting"
+
+count_osts() {
+	do_facet mgs $LCTL get_param mgs.MGS.live.$FSNAME | grep OST | wc -l
+}
+
+test_59() {
+	start_mgsmds >> /dev/null
+	local C1=$(count_osts)
+	if [ $C1 -eq 0 ]; then
+		start_ost >> /dev/null
+		C1=$(count_osts)
+	fi
+	stopall
+	echo "original ost count: $C1 (expect > 0)"
+	[ $C1 -gt 0 ] || error "No OSTs in $FSNAME log"
+	start_mgsmds -o writeconf >> /dev/null || error "MDT start failed"
+	local C2=$(count_osts)
+	echo "after mdt writeconf count: $C2 (expect 0)"
+	[ $C2 -gt 0 ] && error "MDT writeconf should erase OST logs"
+	echo "OST start without writeconf should fail:"
+	start_ost >> /dev/null && error "OST start without writeconf didn't fail"
+	echo "OST start with writeconf should succeed:"
+	start_ost -o writeconf >> /dev/null || error "OST1 start failed"
+	local C3=$(count_osts)
+	echo "after ost writeconf count: $C3 (expect 1)"
+	[ $C3 -eq 1 ] || error "new OST writeconf should add:"
+	start_ost2 -o writeconf >> /dev/null || error "OST2 start failed"
+	local C4=$(count_osts)
+	echo "after ost2 writeconf count: $C4 (expect 2)"
+	[ $C4 -eq 2 ] || error "OST2 writeconf should add log"
+	stop_ost2 >> /dev/null
+	cleanup_nocli >> /dev/null
+}
+run_test 59 "writeconf mount option"
+
 if ! combined_mgs_mds ; then
 	stop mgs
 fi
 
 equals_msg `basename $0`: test complete
 [ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG && grep -q FAIL $TESTSUITELOG && exit 1 || true
+
+
