@@ -13,10 +13,8 @@ init_test_env $@
 ALWAYS_EXCEPT="10 $INSANITY_EXCEPT"
 
 if [ "$FAILURE_MODE" = "HARD" ]; then
-    mixed_ost_devs && CONFIG_EXCEPTIONS="0 2 4 5 6 8" && \
-        echo -n "Several ost services on one ost node are used with FAILURE_MODE=$FAILURE_MODE. " && \
-        echo "Except the tests: $CONFIG_EXCEPTIONS" && \
-        ALWAYS_EXCEPT="$ALWAYS_EXCEPT $CONFIG_EXCEPTIONS"
+        skip_env "$0: is not functional with FAILURE_MODE = HARD, please use recovery-double-scale, bz20407"
+        exit 0
 fi
 
 #
@@ -194,10 +192,10 @@ test_2() {
 
     echo "Reintegrating OST"
     reboot_facet ost1
-    wait_for ost1
+    wait_for_facet ost1
     start_ost 1 || return 2
 
-    wait_for mds
+    wait_for_facet mds
     start mds $MDSDEV $MDS_MOUNT_OPTS || return $?
 
     #Check FS
@@ -270,10 +268,10 @@ test_4() {
     #Reintegration
     echo "Reintegrating OST"
     reboot_facet ost1
-    wait_for ost1
+    wait_for_facet ost1
     start_ost 1
 
-    wait_for mds
+    wait_for_facet mds
     start mds $MDSDEV $MDS_MOUNT_OPTS
     #Check FS
 
@@ -320,9 +318,9 @@ test_5() {
 
     #Reintegration
     echo "Reintegrating OSTs"
-    wait_for ost1
+    wait_for_facet ost1
     start_ost 1
-    wait_for ost2
+    wait_for_facet ost2
     start_ost 2
 
     clients_recover_osts ost1
@@ -371,7 +369,7 @@ test_6() {
 
     #Reintegration
     echo "Reintegrating OST/CLIENTs"
-    wait_for ost1
+    wait_for_facet ost1
     start_ost 1
     reintegrate_clients || return 1
     sleep 5
@@ -491,7 +489,7 @@ test_8() {
     #Reintegration
     echo "Reintegrating CLIENTs/OST"
     reintegrate_clients || return 3
-    wait_for ost1
+    wait_for_facet ost1
     start_ost 1
     wait $DFPID
     clients_up || return 1
