@@ -95,13 +95,12 @@ static char *get_iname(char *fname, const char *mtpt)
 
 int main(int argc, char *argv[])
 {
-        char *fname, *mtpt, *pname;
+        char *fname, *mtpt;
         char *fname_iname, *dir;
         char *dir_iname = NULL, *foo = NULL, *bar = NULL;
         int rc, fd, i, thread = 0;
         int pidlist[10];
 
-        pname = strdup(argv[0]);
         progname = basename(argv[0]);
 
         if (argc != 2) {
