@@ -92,13 +92,13 @@ int main(int argc, char** argv)
         int    fd;
         int    flags = 0;
         mode_t mode = 0644;
-        char*  fname = NULL;
+        char  *fname = NULL;
         int    mode_set = 0;
         int    flag_set = 0;
         int    c;
         int    save_errno = 0;
         int    print_usage = 0;
-        char*  cloned_flags = NULL;
+        char  *cloned_flags = NULL;
 
         if (argc == 1)
                 Usage_and_abort();
@@ -123,9 +123,10 @@ int main(int argc, char** argv)
                                 printf("flags = %d\n",flags);
 #endif
                                 break;
-                        } else 
+                        } else {
                                 flags = 0;
-                        
+                        }
+
                         for (tmp = strtok(cloned_flags, ":|"); tmp;
                              tmp = strtok(NULL, ":|")) {
                                 int i = 0;
