@@ -13,13 +13,6 @@ init_test_env $@
 
 remote_mds_nodsh && skip "remote MDS with nodsh" && exit 0
 
-if [ "$FAILURE_MODE" = "HARD" ] && mixed_ost_devs; then
-    CONFIG_EXCEPTIONS="52"
-    echo -n "Several ost services on one ost node are used with FAILURE_MODE=$FAILURE_MODE. "
-    echo "Except the tests: $CONFIG_EXCEPTIONS"
-    ALWAYS_EXCEPT="$ALWAYS_EXCEPT $CONFIG_EXCEPTIONS"
-fi
-
 # also long tests: 19, 21a, 21e, 21f, 23, 27
 #                                   1  2.5  2.5    4    4          (min)"
 [ "$SLOW" = "no" ] && EXCEPT_SLOW="17  26a  26b    50   51     57"
@@ -300,6 +293,7 @@ test_18b() {
     ost_evict_client
 
     # force reconnect
+    sleep 1
     df $MOUNT > /dev/null 2>&1
     sleep 2
     # my understanding is that there should be nothing in the page
@@ -336,6 +330,7 @@ test_18c() {
     # lost reply to connect request
     do_facet ost1 lctl set_param fail_loc=0x80000225
     # force reconnect
+    sleep 1
     df $MOUNT > /dev/null 2>&1
     sleep 2
     # my understanding is that there should be nothing in the page
@@ -762,12 +757,8 @@ test_27() {
 #define OBD_FAIL_OSC_SHUTDOWN            0x407
 	do_facet mds lctl set_param fail_loc=0x80000407
 	# need to wait for reconnect
-	echo -n waiting for fail_loc
-	while [ $(do_facet mds lctl get_param -n fail_loc) -eq -2147482617 ]; do
-	    sleep 1
-	    echo -n .
-	done
-	do_facet mds lctl get_param -n fail_loc
+	echo waiting for fail_loc
+	wait_update_facet mds "lctl get_param -n fail_loc" "-2147482617"
 	facet_failover mds
 	#no crashes allowed!
         kill -USR1 $CLIENT_PID
