@@ -22,13 +22,6 @@ remote_mds_nodsh && log "SKIP: remote MDS with nodsh" && exit 0
 # bug number:
 ALWAYS_EXCEPT="$REPLAY_SINGLE_EXCEPT"
 
-if [ "$FAILURE_MODE" = "HARD" ] && mixed_ost_devs; then
-    CONFIG_EXCEPTIONS="0b 42 47 61a 61c"
-    echo -n "Several ost services on one ost node are used with FAILURE_MODE=$FAILURE_MODE. "
-    echo "Except the tests: $CONFIG_EXCEPTIONS"
-    ALWAYS_EXCEPT="$ALWAYS_EXCEPT $CONFIG_EXCEPTIONS"
-fi
-
 #                                                  63 min  7 min  AT AT AT AT"
 [ "$SLOW" = "no" ] && EXCEPT_SLOW="1 2 3 4 6 12 16 44a     44b    65 66 67 68"
 
@@ -411,13 +404,8 @@ test_20b() { # bug 10480
     client_up || client_up || true    # reconnect
 
     fail mds                            # start orphan recovery
-    wait_recovery_complete mds || error "MDS recovery not done"
-
-    # For interop with 2.0 only:
-    # FIXME just because recovery is done doesn't mean we've finished
-    # orphan cleanup.  Fake it with a sleep for now...
-    sleep 10
-
+    wait_recovery_complete mds || error "MDS recovery isn't done"
+    wait_mds_ost_sync || return 3
     AFTERUSED=`df -P $DIR | tail -1 | awk '{ print $3 }'`
     log "before $BEFOREUSED, after $AFTERUSED"
     [ $AFTERUSED -gt $((BEFOREUSED + 20)) ] && \
@@ -1061,6 +1049,7 @@ run_test 52 "time out lock replay (3764)"
 
 # bug 3462 - simultaneous MDC requests
 test_53a() {
+    cancel_lru_locks mdc  # lock cleanup
     mkdir -p $DIR/${tdir}-1
     mkdir -p $DIR/${tdir}-2
     multiop $DIR/${tdir}-1/f O_c &
@@ -1088,6 +1077,7 @@ test_53a() {
 run_test 53a "|X| close request while two MDC requests in flight"
 
 test_53b() {
+    cancel_lru_locks mdc  # lock cleanup
     rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
     mkdir -p $DIR/$tdir-1
@@ -1120,6 +1110,7 @@ test_53b() {
 run_test 53b "|X| open request while two MDC requests in flight"
 
 test_53c() {
+    cancel_lru_locks mdc  # lock cleanup
     rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
     mkdir -p $DIR/${tdir}-1
@@ -1153,6 +1144,7 @@ test_53c() {
 run_test 53c "|X| open request and close request while two MDC requests in flight"
 
 test_53d() {
+    cancel_lru_locks mdc  # lock cleanup
     rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
     mkdir -p $DIR/${tdir}-1
@@ -1182,6 +1174,7 @@ test_53d() {
 run_test 53d "|X| close reply while two MDC requests in flight"
 
 test_53e() {
+    cancel_lru_locks mdc  # lock cleanup
     rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
     mkdir -p $DIR/$tdir-1
@@ -1213,6 +1206,7 @@ test_53e() {
 run_test 53e "|X| open reply while two MDC requests in flight"
 
 test_53f() {
+    	cancel_lru_locks mdc  # lock cleanup
         rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
         mkdir -p $DIR/${tdir}-1
@@ -1246,6 +1240,7 @@ test_53f() {
 run_test 53f "|X| open reply and close reply while two MDC requests in flight"
 
 test_53g() {
+    	cancel_lru_locks mdc  # lock cleanup
         rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
         mkdir -p $DIR/${tdir}-1
@@ -1279,6 +1274,7 @@ test_53g() {
 run_test 53g "|X| drop open reply and close request while close and open are both in flight"
 
 test_53h() {
+    cancel_lru_locks mdc  # lock cleanup
     rm -rf $DIR/${tdir}-1 $DIR/${tdir}-2
 
     mkdir -p $DIR/${tdir}-1
@@ -1479,9 +1475,9 @@ test_61d() { # bug 16002
 #define OBD_FAIL_OBD_LLOG_SETUP        0x605
     stop mgs
     do_facet mgs "lctl set_param fail_loc=0x80000605"
-    start mgs $MGSDEV $mgs_MOUNT_OPTS && error "mgs start should have failed"
+    start mgs $MGSDEV $MGS_MOUNT_OPTS && error "mgs start should have failed"
     do_facet mgs "lctl set_param fail_loc=0"
-    start mgs $MGSDEV $mgs_MOUNT_OPTS || error "cannot restart mgs"
+    start mgs $MGSDEV $MGS_MOUNT_OPTS || error "cannot restart mgs"
 }
 run_test 61d "error in llog_setup should cleanup the llog context correctly"
 
@@ -1599,6 +1595,8 @@ test_65b() #bug 3055
     $LCTL dk > /dev/null
     # Slow down a request to the current service time, this is critical
     # because previous tests may have caused this value to increase.
+    lfs setstripe $DIR/$tfile --index=0 --count=1
+    multiop $DIR/$tfile Ow1yc
     REQ_DELAY=`lctl get_param -n osc.${FSNAME}-OST0000-osc-*.timeouts |
                awk '/portal 6/ {print $5}'`
     REQ_DELAY=$((${REQ_DELAY} + ${REQ_DELAY} / 4 + 5))
@@ -1698,7 +1696,7 @@ test_67b() #bug 3055
     CONN1=$(lctl get_param -n osc.*.stats | awk '/_connect/ {total+=$2} END {print total}')
 
     # exhaust precreations on ost1
-    local OST=$(lfs osts | grep 0": " | awk '{print $2}' | sed -e 's/_UUID$//')
+    local OST=$(lfs osts | grep ^0": " | awk '{print $2}' | sed -e 's/_UUID$//')
     local mdtosc=$(get_mdtosc_proc_path $OST)
     local last_id=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_last_id)
     local next_id=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_next_id)
@@ -1796,6 +1794,24 @@ test_70a () {
 }
 run_test 70a "check multi client t-f"
 
+check_dbench_load () {
+	local clients=${1//,/ }
+	local client=
+
+	for client in $clients; do
+		if ! do_node $client "ps ax | grep -v grep | awk '{ print $6 }' | grep -q rundbench"; then
+			error_noexit "rundbench load on $client failed!"
+			return 1
+		fi
+	done
+	return 0
+}
+
+kill_dbench_load () {
+	local clients=${1:-$(hostname)}
+	do_nodes $clients "killall dbench"
+}
+
 test_70b () {
 	local clients=${CLIENTS:-$HOSTNAME}
 
@@ -1817,6 +1833,10 @@ test_70b () {
 	START_TS=$(date +%s)
 	CURRENT_TS=$START_TS
 	while [ $ELAPSED -lt $duration ]; do
+		if ! check_dbench_load $clients; then
+			kill_dbench_load $clients
+			break
+		fi
 		sleep 1
 		replay_barrier mds
 		sleep 1 # give clients a time to do operations
@@ -2105,24 +2125,21 @@ run_test 84 "stale open during export disconnect"
 test_85a() { #bug 16774
     lctl set_param -n ldlm.cancel_unused_locks_before_replay "1"
 
-    for i in `seq 100`; do
-        echo "tag-$i" > $DIR/$tfile-$i
-        grep -q "tag-$i" $DIR/$tfile-$i || error "f2-$i"
-    done
+    createmany -o $DIR/$tfile- 100
+    ls -l $DIR/ > /dev/null
 
     lov_id=`lctl dl | grep "clilov"` 
     addr=`echo $lov_id | awk '{print $4}' | awk -F '-' '{print $3}'`
     count=`lctl get_param -n ldlm.namespaces.*MDT0000*$addr.lock_unused_count`
     echo "before recovery: unused locks count = $count"
+    [ $count -ne 0 ] || error "unused locks should not be zero before recovery"
 
     fail mds
     
     count2=`lctl get_param -n ldlm.namespaces.*MDT0000*$addr.lock_unused_count`
     echo "after recovery: unused locks count = $count2"
 
-    if [ $count2 -ge $count ]; then
-        error "unused locks are not canceled"
-    fi
+    [ $count2 -lt $count ] || error "unused locks are not canceled"
 }
 run_test 85a "check the cancellation of unused locks during recovery(IBITS)"
 
@@ -2159,12 +2176,90 @@ run_test 85b "check the cancellation of unused locks during recovery(EXTENT)"
 
 test_86() {
 	local dev=$(get_mds_mdt_device_proc_path)
-        umount $MOUNT
+	local clients=${CLIENTS:-$HOSTNAME}
+
+	zconf_umount_clients $clients $MOUNT
 	do_facet mds lctl set_param $dev.${FSNAME}-MDT*.exports.clear=0
-	fail mds
+	remount_facet mds
+	zconf_mount_clients $clients $MOUNT
 }
 run_test 86 "umount server after clear nid_stats should not hit LBUG"
 
+test_87() { #bug 17485
+    mkdir -p $DIR/$tdir
+    mkdir -p $TMP/$tdir
+
+    lfs setstripe $DIR/$tdir -o 0 -c 1 || error "setstripe"
+
+    replay_barrier ost1
+    replay_barrier mds
+
+    # exhaust precreations on ost1
+    local OST=$(lfs osts | grep ^0": " | awk '{print $2}' | sed -e 's/_UUID$//')
+    local mdtosc=$(get_mdtosc_proc_path $OST)
+    local last_id=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_last_id)
+    local next_id=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_next_id)
+    echo "before test: last_id = $last_id, next_id = $next_id" 
+
+    echo "Creating to objid $last_id on ost $OST..."
+    createmany -o $DIR/$tdir/f-%d $next_id $((last_id - next_id + 2))
+
+    #create some files to use some uncommitted objids
+    last_id=$(($last_id + 1))
+    createmany -o $DIR/$tdir/f-%d $last_id 8
+
+    last_id2=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_last_id)
+    next_id2=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_next_id)
+    echo "before recovery: last_id = $last_id2, next_id = $next_id2" 
+
+    shutdown_facet mds
+    shutdown_facet ost1
+
+    reboot_facet mds
+    change_active mds
+    wait_for_facet mds
+    mount_facet mds || error "Restart of mds failed"
+
+    reboot_facet ost1
+    change_active ost1
+    wait_for_facet ost1
+    mount_facet ost1 || error "Restart of ost1 failed"
+
+    clients_up
+
+    last_id2=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_last_id)
+    next_id2=$(do_facet mds lctl get_param -n osc.$mdtosc.prealloc_next_id)
+    echo "after recovery: last_id = $last_id2, next_id = $next_id2" 
+
+    # create new files, which should use new objids, and ensure the orphan 
+    # cleanup phase for ost1 is completed at the same time
+    for i in `seq 8`; do
+        file_id=$(($last_id + 10 + $i))
+        dd if=/dev/urandom of=$DIR/$tdir/f-$file_id bs=4096 count=128
+    done
+
+    # if the objids were not recreated, then "ls" will failed for -ENOENT
+    ls -l $DIR/$tdir/* || error "can't get the status of precreated files"
+
+    local file_id
+    # write into previously created files
+    for i in `seq 8`; do
+        file_id=$(($last_id + $i))
+        dd if=/dev/urandom of=$DIR/$tdir/f-$file_id bs=4096 count=128
+        cp -f $DIR/$tdir/f-$file_id $TMP/$tdir/
+    done
+
+    # compare the content
+    for i in `seq 8`; do
+        file_id=$(($last_id + $i))
+        cmp $TMP/$tdir/f-$file_id $DIR/$tdir/f-$file_id || error "the content" \
+        "of file is modified!"
+    done
+
+    rm -fr $TMP/$tdir
+}
+run_test 87 "MDS should not assign same objid to different files "
+
 equals_msg `basename $0`: test complete, cleaning up
 check_and_cleanup_lustre
 [ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG && grep -q FAIL $TESTSUITELOG && exit 1 || true
