@@ -99,6 +99,23 @@ init_test_env() {
     export E2LABEL=${E2LABEL:-e2label}
     export DUMPE2FS=${DUMPE2FS:-dumpe2fs}
     export E2FSCK=${E2FSCK:-e2fsck}
+    export LFSCK_BIN=${LFSCK_BIN:-lfsck}
+    export LFSCK_ALWAYS=${LFSCK_ALWAYS:-"no"} # check filesystem after each test suit
+    export SKIP_LFSCK=${SKIP_LFSCK:-"yes"} # bug 13698, change to "no" when fixed
+    export SHARED_DIRECTORY=${SHARED_DIRECTORY:-"/tmp"}
+    export FSCK_MAX_ERR=4   # File system errors left uncorrected
+    if [ "$SKIP_LFSCK" == "no" ]; then
+	if [ ! -x `which $LFSCK_BIN` ]; then
+	    log "$($E2FSCK -V)"
+	    error_exit "$E2FSCK does not support lfsck"
+	fi
+
+	export MDSDB=${MDSDB:-$SHARED_DIRECTORY/mdsdb}
+	export OSTDB=${OSTDB:-$SHARED_DIRECTORY/ostdb}
+	export MDSDB_OPT="--mdsdb $MDSDB"
+	export OSTDB_OPT="--ostdb $OSTDB-\$ostidx"
+    fi
+    declare -a OSTDEVS
 
     #[ -d /r ] && export ROOT=${ROOT:-/r}
     export TMP=${TMP:-$ROOT/tmp}
@@ -112,6 +129,7 @@ init_test_env() {
     fi
     export LST=${LST:-"$LUSTRE/../lnet/utils/lst"}
     [ ! -f "$LST" ] && export LST=$(which lst)
+    export SGPDDSURVEY=${SGPDDSURVEY:-$(which sgpdd-survey)}
     export MDSRATE=${MDSRATE:-"$LUSTRE/tests/mpi/mdsrate"}
     [ ! -f "$MDSRATE" ] && export MDSRATE=$(which mdsrate 2> /dev/null)
     if ! echo $PATH | grep -q $LUSTRE/tests/racer; then
@@ -122,7 +140,6 @@ init_test_env() {
     fi
     export RSYNC_RSH=${RSYNC_RSH:-rsh}
     export LCTL=${LCTL:-"$LUSTRE/utils/lctl"}
-    export LFS=${LFS:-"$LUSTRE/utils/lfs"}
     [ ! -f "$LCTL" ] && export LCTL=$(which lctl)
     export LFS=${LFS:-"$LUSTRE/utils/lfs"}
     [ ! -f "$LFS" ] && export LFS=$(which lfs)
@@ -149,6 +166,7 @@ init_test_env() {
     export RPWD=${RPWD:-$PWD}
     export I_MOUNTED=${I_MOUNTED:-"no"}
     if [ ! -f /lib/modules/$(uname -r)/kernel/fs/lustre/mds.ko -a \
+        ! -f /lib/modules/$(uname -r)/updates/kernel/fs/lustre/mds.ko -a \
         ! -f `dirname $0`/../mds/mds.ko ]; then
         export CLIENTMODSONLY=yes
     fi
@@ -274,7 +292,7 @@ load_modules_local() {
     load_module lvfs/lvfs
     load_module obdclass/obdclass
     load_module ptlrpc/ptlrpc
-    [ "$USE_QUOTA" = "yes" ] && load_module quota/lquota
+    [ "$USE_QUOTA" = "yes" ] && load_module quota/lquota $LQUOTAOPTS
     load_module mdc/mdc
     load_module osc/osc
     load_module lov/lov
@@ -693,12 +711,18 @@ fi
 fi"
 }
 
-shudown_node_hard () {
+shutdown_node () {
+    local node=$1
+    echo + $POWER_DOWN $node
+    $POWER_DOWN $node
+}
+
+shutdown_node_hard () {
     local host=$1
     local attempts=3
 
     for i in $(seq $attempts) ; do
-        $POWER_DOWN $host
+        shutdown_node $host
         sleep 1
         ping -w 3 -c 1 $host > /dev/null 2>&1 || return 0
         echo "waiting for $host to fail attempts=$attempts"
@@ -713,25 +737,55 @@ shutdown_client() {
     local attempts=3
 
     if [ "$FAILURE_MODE" = HARD ]; then
-        shudown_node_hard $client
+        shutdown_node_hard $client
     else
        zconf_umount_clients $client $mnt -f
     fi
 }
 
+facets_on_host () {
+    local host=$1
+    local facets="$(get_facets OST),$(get_facets MDS)"
+    local affected
+
+    combined_mgs_mds || facets="$facets,mgs"
+
+    for facet in ${facets//,/ }; do
+        if [ $(facet_active_host $facet) == $host ]; then
+           affected="$affected $facet"
+        fi
+    done
+
+    echo $(comma_list $affected)
+}
+
 shutdown_facet() {
     local facet=$1
+
     if [ "$FAILURE_MODE" = HARD ]; then
-        shudown_node_hard $(facet_active_host $facet)
-    elif [ "$FAILURE_MODE" = SOFT ]; then
+        shutdown_node_hard $(facet_active_host $facet)
+    else
         stop $facet
     fi
 }
 
+reboot_node() {
+    local node=$1
+    echo + $POWER_UP $node
+    $POWER_UP $node
+}
+
+remount_facet() {
+    local facet=$1
+
+    stop $facet
+    mount_facet $facet
+}
+
 reboot_facet() {
-    facet=$1
+    local facet=$1
     if [ "$FAILURE_MODE" = HARD ]; then
-        $POWER_UP `facet_active_host $facet`
+        reboot_node $(facet_active_host $facet)
     else
         sleep 10
     fi
@@ -740,7 +794,7 @@ reboot_facet() {
 boot_node() {
     local node=$1
     if [ "$FAILURE_MODE" = HARD ]; then
-       $POWER_UP $node
+       reboot_node $node
        wait_for_host $node
     fi
 }
@@ -956,48 +1010,129 @@ wait_delete_completed () {
         sleep 1
         TOTAL=`lctl get_param -n osc.*.kbytesavail | \
                awk 'BEGIN{total=0}; {total+=$1}; END{print total}'`
-        [ "$TOTAL" -eq "$TOTALPREV" ] && break
+        [ "$TOTAL" -eq "$TOTALPREV" ] && return 0
         echo "Waiting delete completed ... prev: $TOTALPREV current: $TOTAL "
         TOTALPREV=$TOTAL
         WAIT=$(( WAIT + 1))
     done
-    echo "Delete completed."
+    echo "Delete is not completed in $MAX_WAIT sec"
+    return 1
 }
 
 wait_for_host() {
-    local host=$1
-    check_network "$host" 900
-    while ! do_node $host hostname  > /dev/null; do sleep 5; done
+    local hostlist=$1
+
+    # we can use "for" here because we are waiting the slowest
+    for host in ${hostlist//,/ }; do
+        check_network "$host" 900
+    done
+    while ! do_nodes $hostlist hostname  > /dev/null; do sleep 5; done
 }
 
-wait_for() {
-    local facet=$1
-    local host=`facet_active_host $facet`
-    wait_for_host $host
+wait_for_facet() {
+    local facetlist=$1
+    local hostlist
+
+    for facet in ${facetlist//,/ }; do
+        hostlist=$(expand_list $hostlist $(facet_active_host $facet))
+    done
+    wait_for_host $hostlist
+}
+
+_wait_recovery_complete () {
+    local param=$1
+
+    # Use default policy if $2 is not passed by caller.
+    local MAX=${2:-$(max_recovery_time)}
+
+    local WAIT=0
+    local STATUS=
+
+    while [ $WAIT -lt $MAX ]; do
+        STATUS=$(lctl get_param -n $param | grep status)
+        echo $param $STATUS
+        [[ $STATUS = "status: COMPLETE" || $STATUS = "status: INACTIVE" ]] && return 0
+        sleep 5
+        WAIT=$((WAIT + 5))
+        echo "Waiting $((MAX - WAIT)) secs for $param recovery done. $STATUS"
+    done
+    echo "$param recovery not done in $MAX sec. $STATUS"
+    return 1
 }
 
 wait_recovery_complete () {
     local facet=$1
 
-    # Use default policy if $2 is not passed by caller.
+    # with an assumption that at_max is the same on all nodes
     local MAX=${2:-$(max_recovery_time)}
 
-    local var_svc=${facet}_svc
-    local procfile="*.${!var_svc}.recovery_status"
-    local WAIT=0
-    local STATUS=
+    local facets=$facet
+    if [ "$FAILURE_MODE" = HARD ]; then
+        facets=$(facets_on_host $(facet_active_host $facet))
+    fi
+    echo affected facets: $facets
 
+    # we can use "for" here because we are waiting the slowest
+    for facet in ${facets//,/ }; do
+        local var_svc=${facet}_svc
+        local param="*.${!var_svc}.recovery_status"
+
+        local host=$(facet_active_host $facet)
+        do_rpc_nodes $host _wait_recovery_complete $param $MAX
+    done
+}
+
+wait_mds_ost_sync () {
+    # just because recovery is done doesn't mean we've finished
+    # orphan cleanup. Wait for llogs to get synchronized.
+    echo "Waiting for orphan cleanup..."
+    # MAX value includes time needed for MDS-OST reconnection
+    local MAX=$(( TIMEOUT * 2 ))
+    local WAIT=0
     while [ $WAIT -lt $MAX ]; do
-        STATUS=$(do_facet $facet lctl get_param -n $procfile | grep status)
-        [[ $STATUS = "status: COMPLETE" ]] && return 0
-        sleep 5
-        WAIT=$((WAIT + 5))
-        echo "Waiting $((MAX - WAIT)) secs for $facet recovery done. $STATUS"
+        local -a sync=($(do_nodes $(comma_list $(osts_nodes)) \
+            "$LCTL get_param -n obdfilter.*.mds_sync"))
+        local con=1
+        for ((i=0; i<${#sync[@]}; i++)); do
+            [ ${sync[$i]} -eq 0 ] && continue
+            # there is a not finished MDS-OST synchronization
+            con=0
+            break;
+        done
+        sleep 2 # increase waiting time and cover statfs cache
+        [ ${con} -eq 1 ] && return 0
+        echo "Waiting $WAIT secs for $facet mds-ost sync done."
+        WAIT=$((WAIT + 2))
     done
     echo "$facet recovery not done in $MAX sec. $STATUS"
     return 1
 }
 
+wait_destroy_complete () {
+    echo "Waiting for destroy to be done..."
+    # MAX value shouldn't be big as this mean server responsiveness
+    # never increase this just to make test pass but investigate
+    # why it takes so long time
+    local MAX=5
+    local WAIT=0
+    while [ $WAIT -lt $MAX ]; do
+        local -a RPCs=($($LCTL get_param -n osc.*.destroys_in_flight))
+        local con=1
+        for ((i=0; i<${#RPCs[@]}; i++)); do
+            [ ${RPCs[$i]} -eq 0 ] && continue
+            # there are still some destroy RPCs in flight
+            con=0
+            break;
+        done
+        sleep 1
+        [ ${con} -eq 1 ] && return 0 # done waiting
+        echo "Waiting $WAIT secs for destroys to be done."
+        WAIT=$((WAIT + 1))
+    done
+    echo "Destroys weren't done in $MAX sec."
+    return 1
+}
+
 wait_exit_ST () {
     local facet=$1
 
@@ -1092,19 +1227,40 @@ client_reconnect() {
 facet_failover() {
     local facet=$1
     local sleep_time=$2
-    echo "Failing $facet on node `facet_active_host $facet`"
+    local host=$(facet_active_host $facet)
+
+    echo "Failing $facet on node $host"
+
+    local affected=$facet
+
+    if [ "$FAILURE_MODE" = HARD ]; then
+        affected=$(facets_on_host $host)
+    fi
+
     shutdown_facet $facet
+
+    echo affected facets: $affected
+
     [ -n "$sleep_time" ] && sleep $sleep_time
+
     reboot_facet $facet
     clients_up &
-    DFPID=$!
+    local dfpid=$!
     RECOVERY_START_TIME=`date +%s`
-    echo "df pid is $DFPID"
-    change_active $facet
-    local TO=`facet_active_host $facet`
-    echo "Failover $facet to $TO"
-    wait_for $facet
-    mount_facet $facet || error "Restart of $facet failed"
+    echo "df pid is $dfpid"
+
+    change_active $affected
+
+    wait_for_facet $affected
+    # start mgs first if it is affected
+    if ! combined_mgs_mds && list_member $affected mgs; then
+        mount_facet mgs || error "Restart of mgs failed"
+    fi
+    # FIXME; has to be changed to mount all facets concurrently 
+    affected=$(exclude_items_from_list $affected mgs)
+    for facet in ${affected//,/ }; do
+        mount_facet $facet || error "Restart of $facet on node $host failed!"
+    done
 }
 
 obd_name() {
@@ -1265,10 +1421,16 @@ facet_active_host() {
 }
 
 change_active() {
-    local facet=$1
+    local facetlist=$1
+    local facet
+
+    facetlist=$(exclude_items_from_list $facetlist mgs)
+
+    for facet in ${facetlist//,/ }; do
     local failover=${facet}failover
-    host=`facet_host $failover`
+    local host=`facet_host $failover`
     [ -z "$host" ] && return
+
     local curactive=`facet_active $facet`
     if [ -z "${curactive}" -o "$curactive" == "$failover" ] ; then
         eval export ${facet}active=$facet
@@ -1278,6 +1440,9 @@ change_active() {
     # save the active host for this facet
     local activevar=${facet}active
     echo "$activevar=${!activevar}" > $TMP/$activevar
+    local TO=`facet_active_host $facet`
+    echo "Failover $facet to $TO"
+    done
 }
 
 do_node() {
@@ -1365,10 +1530,12 @@ do_nodes() {
         $myPDSH $rnodes $LCTL mark "$@" > /dev/null 2>&1 || :
     fi
 
-    if $verbose ; then
+    # do not replace anything from pdsh output if -N is used
+    # -N     Disable hostname: prefix on lines of output.
+    if $verbose || [[ $myPDSH = *-N* ]]; then
         $myPDSH $rnodes "(PATH=\$PATH:$RLUSTRE/utils:$RLUSTRE/tests:/sbin:/usr/sbin; cd $RPWD; sh -c \"$@\")"
     else
-        $myPDSH $rnodes "(PATH=\$PATH:$RLUSTRE/utils:$RLUSTRE/tests:/sbin:/usr/sbin; cd $RPWD; sh -c \"$@\")" | sed -re "s/\w+:\s//g"
+        $myPDSH $rnodes "(PATH=\$PATH:$RLUSTRE/utils:$RLUSTRE/tests:/sbin:/usr/sbin; cd $RPWD; sh -c \"$@\")" | sed -re "s/^[^:]*: //g"
     fi
     return ${PIPESTATUS[0]}
 }
@@ -1447,10 +1614,21 @@ stopall() {
     return 0
 }
 
+cleanup_echo_devs () {
+    local devs=$($LCTL dl | grep echo | awk '{print $4}')
+
+    for dev in $devs; do
+        $LCTL --device $dev cleanup
+        $LCTL --device $dev detach
+    done
+}
+
 cleanupall() {
     nfs_client_mode && return
 
     stopall $*
+    cleanup_echo_devs
+
     unload_modules
 }
 
@@ -1458,6 +1636,53 @@ combined_mgs_mds () {
     [[ $MDSDEV = $MGSDEV ]] && [[ $mds_HOST = $mgs_HOST ]]
 }
 
+mkfs_opts () {
+    local facet=$1
+
+    local tgt=$(echo $facet | tr -d [:digit:] | tr "[:lower:]" "[:upper:]")
+    local optvar=${tgt}_MKFS_OPTS
+    local opt=${!optvar}
+
+    # FIXME: ! combo  mgs/mds + mgsfailover is not supported yet
+    [[ $facet = mgs ]] && echo $opt && return
+
+    # 1.
+    # --failnode options 
+    local var=${facet}failover_HOST
+    if [ x"${!var}" != x ] && [ x"${!var}" != x$(facet_host $facet) ] ; then
+        local failnode=$(h2$NETTYPE ${!var})
+        failnode="--failnode=$failnode"
+        # options does not contain 
+        # or contains wrong --failnode=
+        if [[ $opt != *${failnode}* ]]; then
+            opt=$(echo $opt | sed 's/--failnode=.* / /')
+            opt="$opt $failnode"
+        fi
+    fi
+
+    # 2.
+    # --mgsnode options
+    # no additional mkfs mds "--mgsnode" option for this configuration
+    if [[ $facet = mds ]] && combined_mgs_mds; then
+        echo $opt
+        return
+    fi
+
+    # additional mkfs "--mgsnode"
+    local mgsnode="--mgsnode=$MGSNID"
+    opt=${opt//$mgsnode }
+    for nid in ${MGSNID//:/ }; do
+        local mgsnode="--mgsnode=$nid"
+        # options does not contain
+        # --mgsnode=$nid
+        if [[ $opt != *${mgsnode}" "* ]]; then
+            opt="$opt --mgsnode=$nid"
+        fi
+    done
+
+    echo $opt
+}
+
 formatall() {
     [ "$FSTYPE" ] && FSTYPE_OPT="--backfstype $FSTYPE"
 
@@ -1467,20 +1692,22 @@ formatall() {
     [ "$CLIENTONLY" ] && return
     echo Formatting mgs, mds, osts
     if ! combined_mgs_mds ; then
-        add mgs $mgs_MKFS_OPTS $FSTYPE_OPT --reformat $MGSDEV || exit 10
+        add mgs $(mkfs_opts mgs) $FSTYPE_OPT --reformat $MGSDEV || exit 10
     fi
 
     if $VERBOSE; then
-        add mds $MDS_MKFS_OPTS $FSTYPE_OPT --reformat $MDSDEV || exit 10
+        add mds $(mkfs_opts mds) $FSTYPE_OPT --reformat $MDSDEV || exit 10
     else
-        add mds $MDS_MKFS_OPTS $FSTYPE_OPT --reformat $MDSDEV > /dev/null || exit 10
+        add mds $(mkfs_opts mds) $FSTYPE_OPT --reformat $MDSDEV > /dev/null || exit 10
     fi
 
+    # the ost-s could have different OST_MKFS_OPTS
+    # because of different failnode-s
     for num in `seq $OSTCOUNT`; do
         if $VERBOSE; then
-            add ost$num $OST_MKFS_OPTS $FSTYPE_OPT --reformat `ostdevname $num` || exit 10
+            add ost$num $(mkfs_opts ost${num}) $FSTYPE_OPT --reformat `ostdevname $num` || exit 10
         else
-            add ost$num $OST_MKFS_OPTS $FSTYPE_OPT --reformat `ostdevname $num` > /dev/null || exit 10
+            add ost$num $(mkfs_opts ost${num}) $FSTYPE_OPT --reformat `ostdevname $num` > /dev/null || exit 10
         fi
     done
 }
@@ -1525,7 +1752,7 @@ setupall() {
             writeconf_all
 
         if ! combined_mgs_mds ; then
-            start mgs $MGSDEV $mgs_MOUNT_OPTS
+            start mgs $MGSDEV $MGS_MOUNT_OPTS
         fi
 
         start mds $MDSDEV $MDS_MOUNT_OPTS
@@ -1610,6 +1837,8 @@ init_facets_vars () {
     remote_mds_nodsh ||
         init_facet_vars mds $MDSDEV $MDS_MOUNT_OPTS
 
+    combined_mgs_mds || init_facet_vars mgs $MGSDEV $MGS_MOUNT_OPTS
+
     remote_ost_nodsh && return
 
     for num in `seq $OSTCOUNT`; do
@@ -1720,6 +1949,20 @@ is_mounted () {
     echo $mounted' ' | grep -w -q $mntpt' '
 }
 
+is_empty_dir() {
+    [ $(find $1 -maxdepth 1 -print | wc -l) = 1 ] && return 0
+    return 1
+}
+
+# empty lustre filesystem may have empty directories lost+found and .lustre
+is_empty_fs() {
+    [ $(find $1 -maxdepth 1 -name lost+found -o -name .lustre -prune -o \
+       -print | wc -l) = 1 ] || return 1
+    [ ! -d $1/lost+found ] || is_empty_dir $1/lost+found && return 0
+    [ ! -d $1/.lustre ] || is_empty_dir $1/.lustre && return 0
+    return 1
+}
+
 check_and_setup_lustre() {
     nfs_client_mode && return
 
@@ -1810,7 +2053,117 @@ cleanup_and_setup_lustre() {
     check_and_setup_lustre
 }
 
+# Get all of the server target devices from a given server node and type.
+get_mnt_devs() {
+    local node=$1
+    local type=$2
+    local obd_type
+    local devs
+    local dev
+
+    case $type in
+    mdt) obd_type="osd" ;;
+    ost) obd_type="obdfilter" ;; # needs to be fixed when OST also uses an OSD
+    *) echo "invalid server type" && return 1 ;;
+    esac
+
+    devs=$(do_node $node "lctl get_param -n $obd_type.*.mntdev")
+    for dev in $devs; do
+        case $dev in
+        *loop*) do_node $node "losetup $dev" | \
+                sed -e "s/.*(//" -e "s/).*//" ;;
+        *) echo $dev ;;
+        esac
+    done
+}
+
+# Get all of the server target devices.
+get_svr_devs() {
+    local i
+
+    # OST devices
+    i=0
+    for node in $(osts_nodes); do
+        OSTDEVS[i]=$(get_mnt_devs $node ost)
+        i=$((i + 1))
+    done
+}
+
+# Run e2fsck on MDT or OST device.
+run_e2fsck() {
+    local node=$1
+    local target_dev=$2
+    local ostidx=$3
+    local ostdb_opt=$4
+
+    df > /dev/null	# update statfs data on disk
+    local cmd="$E2FSCK -d -v -f -n $MDSDB_OPT $ostdb_opt $target_dev"
+    echo $cmd
+    local rc=0
+    do_node $node $cmd || rc=$?
+    [ $rc -le $FSCK_MAX_ERR ] || \
+        error_exit "$cmd returned $rc, should be <= $FSCK_MAX_ERR"
+    return 0
+}
+
+# Run e2fsck on MDT and OST(s) to generate databases used for lfsck.
+generate_db() {
+    local i
+    local ostidx
+    local dev
+    local tmp_file
+
+    tmp_file=$(mktemp -p $SHARED_DIRECTORY || 
+        error_exit "fail to create file in $SHARED_DIRECTORY")
+
+    # make sure everything gets to the backing store
+    local list=$(comma_list $CLIENTS $(facet_host mds) $(osts_nodes))
+    do_nodes $list "sync; sleep 2; sync"
+
+    do_nodes $list ls $tmp_file || \
+        error_exit "$SHARED_DIRECTORY is not a shared directory"
+    rm $tmp_file
+
+    run_e2fsck $(facet_host mds) $MDSDEV
+
+    i=0
+    ostidx=0
+    OSTDB_LIST=""
+    for node in $(osts_nodes); do
+        for dev in ${OSTDEVS[i]}; do
+	    local ostdb_opt=`eval echo $OSTDB_OPT`
+            run_e2fsck $node $dev $ostidx "$ostdb_opt"
+            OSTDB_LIST="$OSTDB_LIST $OSTDB-$ostidx"
+            ostidx=$((ostidx + 1))
+        done
+        i=$((i + 1))
+    done
+}
+
+run_lfsck() {
+    local cmd="$LFSCK_BIN -c -l --mdsdb $MDSDB --ostdb $OSTDB_LIST $MOUNT"
+    echo $cmd
+    local rc=0
+    eval $cmd || rc=$?
+    [ $rc -le $FSCK_MAX_ERR ] || \
+        error_exit "$cmd returned $rc, should be <= $FSCK_MAX_ERR"
+    echo "lfsck finished with rc=$rc"
+
+    rm -rvf $MDSDB* $OSTDB* || true
+    return 0
+}
+
 check_and_cleanup_lustre() {
+    if [ "$LFSCK_ALWAYS" = "yes" ]; then
+        get_svr_devs
+        generate_db
+        if [ "$SKIP_LFSCK" == "no" ]; then
+	    run_lfsck
+	else
+	    echo "skip lfsck"
+        fi
+    fi
+
     if is_mounted $MOUNT; then
         [ -n "$DIR" ] && rm -rf $DIR/[Rdfs][0-9]*
         [ "$ENABLE_QUOTA" ] && restore_quota_type || true
@@ -1868,6 +2221,12 @@ comma_list() {
     echo "$*" | tr -s " " "\n" | sort -b -u | tr "\n" " " | sed 's/ \([^$]\)/,\1/g'
 }
 
+list_member () {
+    local list=$1
+    local item=$2
+    echo $list | grep -qw $item  
+}
+
 # list, excluded are the comma separated lists
 exclude_items_from_list () {
     local list=$1
@@ -1916,19 +2275,22 @@ absolute_path() {
 }
 
 get_facets () {
-    local name=$(echo $1 | tr "[:upper:]" "[:lower:]")
-    local type=$(echo $1 | tr "[:lower:]" "[:upper:]")
+    local types=${1:-"OST MDS MGS"}
 
     local list=""
+    for entry in $types; do
+        local name=$(echo $entry | tr "[:upper:]" "[:lower:]")
+        local type=$(echo $entry | tr "[:lower:]" "[:upper:]")
 
-    case $type in
-        MDS )    list=mds;;
-        OST )    for ((i=1; i<=$OSTCOUNT; i++)) do
-                    list="$list ${name}$i"
-                 done;;
-          * )    error "Invalid facet type"
+        case $type in
+            MDS|MGS ) list="$list $name";;
+                OST ) for ((i=1; i<=$OSTCOUNT; i++)) do
+                          list="$list ${name}$i"
+                      done;;
+                  * ) error "Invalid facet type"
                  exit 1;;
-    esac
+        esac
+    done
     echo $(comma_list $list)
 }
 
@@ -2115,6 +2477,8 @@ debugrestore() {
 error_noexit() {
     local TYPE=${TYPE:-"FAIL"}
     local ERRLOG
+    local tmp=$TMP
+    [ -d "$SHARED_DIR_LOGS" ] && tmp=$SHARED_DIR_LOGS
 
     local dump=true
     # do not dump logs if $1=false
@@ -2126,7 +2490,7 @@ error_noexit() {
     log " ${TESTSUITE} ${TESTNAME}: @@@@@@ ${TYPE}: $@ "
 
     if $dump; then
-        ERRLOG=$TMP/lustre_${TESTSUITE}_${TESTNAME}.$(date +%s)
+        ERRLOG=$tmp/lustre_${TESTSUITE}_${TESTNAME}.$(date +%s)
         echo "Dumping lctl log to $ERRLOG"
         # We need to dump the logs on all nodes
         do_nodes $(comma_list $(nodes_list)) $NODE $LCTL dk $ERRLOG
@@ -2336,12 +2700,6 @@ canonical_path() {
     (cd `dirname $1`; echo $PWD/`basename $1`)
 }
 
-sync_clients() {
-    [ -d $DIR1 ] && cd $DIR1 && sync; sleep 1; sync
-    [ -d $DIR2 ] && cd $DIR2 && sync; sleep 1; sync
-	cd $SAVE_PWD
-}
-
 check_grant() {
     export base=`basetest $1`
     [ "$CHECK_GRANT" == "no" ] && return 0
@@ -2350,39 +2708,33 @@ check_grant() {
     [ ${!testname}x == x ] && return 0
 
     echo -n "checking grant......"
-	cd $SAVE_PWD
-	# write some data to sync client lost_grant
-	rm -f $DIR1/${tfile}_check_grant_* 2>&1
-	for i in `seq $OSTCOUNT`; do
-		$LFS setstripe $DIR1/${tfile}_check_grant_$i -i $(($i -1)) -c 1
-		dd if=/dev/zero of=$DIR1/${tfile}_check_grant_$i bs=4k \
-					      count=1 > /dev/null 2>&1
-	done
+
+    local clients=$CLIENTS
+    [ -z $clients ] && clients=$(hostname)
+
     # sync all the data and make sure no pending data on server
-    sync_clients
+    do_nodes $clients sync
 
-    #get client grant and server grant
-    client_grant=0
-    for d in `lctl get_param -n osc.*.cur_grant_bytes`; do
-        client_grant=$((client_grant + $d))
-    done
-    server_grant=0
-    for d in `lctl get_param -n obdfilter.*.tot_granted`; do
-        server_grant=$((server_grant + $d))
-    done
+    # get client grant
+    client_grant=`do_nodes $clients \
+                    "$LCTL get_param -n osc.${FSNAME}-*.cur_*grant_bytes" | \
+                    awk '{total += $1} END{print total}'`
 
-	# cleanup the check_grant file
-	for i in `seq $OSTCOUNT`; do
-	        rm $DIR1/${tfile}_check_grant_$i
-	done
+    # get server grant
+    server_grant=`do_nodes $(comma_list $(osts_nodes)) \
+                    "$LCTL get_param -n obdfilter.${FSNAME}-OST*.tot_granted" | \
+                    awk '{total += $1} END{print total}'`
 
-	#check whether client grant == server grant
-	if [ $client_grant != $server_grant ]; then
-		echo "failed: client:${client_grant} server: ${server_grant}"
-		return 1
-	else
-		echo "pass"
-	fi
+    # check whether client grant == server grant
+    if [ $client_grant -ne $server_grant ]; then
+        echo "failed: client:${client_grant} server: ${server_grant}."
+        do_nodes $(comma_list $(osts_nodes)) \
+                   "$LCTL get_param obdfilter.${FSNAME}-OST*.tot*"
+        do_nodes $clients "$LCTL get_param osc.${FSNAME}-*.cur_*_bytes"
+        return 1
+    else
+        echo "pass: client:${client_grant} server: ${server_grant}"
+    fi
 }
 
 ########################
@@ -2398,6 +2750,11 @@ osc_to_ost()
     echo $ost
 }
 
+ostuuid_from_index()
+{
+    $LFS osts $2 | awk '/^'$1'/ { print $2 }'
+}
+
 remote_node () {
     local node=$1
     [ "$node" != "$(hostname)" ]
@@ -2446,19 +2803,30 @@ local_mode ()
         $(single_local_node $(comma_list $(nodes_list)))
 }
 
-osts_nodes () {
-    local OSTNODES=$(facet_host ost1)
+facets_nodes () {
+    local facets=$1
+    local nodes
     local NODES_sort
 
-    for num in `seq $OSTCOUNT`; do
-        local myOST=$(facet_host ost$num)
-        OSTNODES="$OSTNODES $myOST"
+    for facet in ${facets//,/ }; do
+        if [ "$FAILURE_MODE" = HARD ]; then
+            nodes="$nodes $(facet_active_host $facet)"
+        else
+            nodes="$nodes $(facet_host $facet)"
+        fi
     done
-    NODES_sort=$(for i in $OSTNODES; do echo $i; done | sort -u)
+    NODES_sort=$(for i in $nodes; do echo $i; done | sort -u)
 
     echo $NODES_sort
 }
 
+osts_nodes () {
+    local facets=$(get_facets OST)
+    local nodes=$(facets_nodes $facets)
+
+    echo $nodes
+}
+
 nodes_list () {
     # FIXME. We need a list of clients
     local myNODES=$HOSTNAME
@@ -2468,7 +2836,7 @@ nodes_list () {
     [ -n "$CLIENTS" ] && myNODES=${CLIENTS//,/ }
 
     if [ "$PDSH" -a "$PDSH" != "no_dsh" ]; then
-        myNODES="$myNODES $(osts_nodes) $mds_HOST"
+        myNODES="$myNODES $(facets_nodes $(get_facets))"
     fi
 
     myNODES_sort=$(for i in $myNODES; do echo $i; done | sort -u)
@@ -2649,6 +3017,10 @@ inodes_available () {
     echo $IFree
 }
 
+mdsrate_inodes_available () {
+    echo $(($(inodes_available) - 1))
+}
+
 # reset llite stat counters
 clear_llite_stats(){
         lctl set_param -n llite.*.stats 0
@@ -2760,13 +3132,28 @@ get_mds_version_fix () {
     echo $version | awk -F. '{print $4}'
 }
 
+get_kernel_version () {
+    local facet=${1:-"mds"}
+    do_facet $facet uname -r | awk -F "-" '{print $1}'
+}
+
+get_mds_kernel_major () {
+    local version=$(get_kernel_version mds)
+    echo $version | awk -F. '{print $1"."$2}'
+}
+
+get_mds_kernel_minor () {
+    local version=$(get_kernel_version mds)
+    echo $version | awk -F. '{print $3}'
+}
+
 get_mds_fsstat_proc_path() {
     local major=$(get_mds_version_major)
     local minor=$(get_mds_version_minor)
     if [ $major -le 1 -a $minor -le 8 ] ; then
         echo "mds"
     else
-        echo "osd"
+        echo "osd*"
     fi
 }
 
@@ -2838,7 +3225,7 @@ convert_facet2label() {
 get_clientosc_proc_path() {
     local ost=$1
 
-    echo "{$1}-osc-*"
+    echo "${1}-osc-*"
 }
 
 get_osc_import_name() {
@@ -2855,33 +3242,37 @@ get_osc_import_name() {
     return 0
 }
 
-wait_import_state () {
+_wait_import_state () {
     local expected=$1
     local CONN_PROC=$2
+    local maxtime=${3:-max_recovery_time}
     local CONN_STATE
     local i=0
 
     CONN_STATE=$($LCTL get_param -n $CONN_PROC 2>/dev/null | cut -f2)
     while [ "${CONN_STATE}" != "${expected}" ]; do
-        if [ "${expected}" == "DISCONN" ]; then
-            # for disconn we can check after proc entry is removed
-            [ "x${CONN_STATE}" == "x" ] && return 0
-            #  with AT we can have connect request timeout ~ reconnect timeout
-            # and test can't see real disconnect
-            [ "${CONN_STATE}" == "CONNECTING" ] && return 0
-        fi
-        # disconnect rpc should be wait not more obd_timeout
-        [ $i -ge $(($TIMEOUT * 3 / 2)) ] && \
-            error "can't put import for $CONN_PROC into ${expected} state" && return 1
+        [ $i -ge $maxtime ] && \
+            error "can't put import for $CONN_PROC into ${expected} state after $i sec, have ${CONN_STATE}" && \
+            return 1
         sleep 1
         CONN_STATE=$($LCTL get_param -n $CONN_PROC 2>/dev/null | cut -f2)
         i=$(($i + 1))
     done
 
-    log "$CONN_PROC now in ${CONN_STATE} state"
+    log "$CONN_PROC in ${CONN_STATE} state after $i sec"
     return 0
 }
 
+wait_import_state() {
+    local state=$1
+    local params=$2
+    local maxtime=${3:-max_recovery_time}
+    local param
+
+    for param in ${params//,/ }; do
+        _wait_import_state $state $param $maxtime || return
+    done
+}
 wait_osc_import_state() {
     local facet=$1
     local ost_facet=$2
@@ -2905,7 +3296,7 @@ wait_osc_import_state() {
         [ $i -ge $(($TIMEOUT * 3 / 2)) ] && \
             error "can't put import for ${ost}(${ost_facet}) into ${expected} state" && return 1
         sleep 1
-        CONN_STATE=$(do_facet $facet lctl get_param -n $CONN_PROC | cut -f2)
+        CONN_STATE=$(do_facet $facet lctl get_param -n $CONN_PROC 2>/dev/null | cut -f2)
         i=$(($i + 1))
     done
 
@@ -2921,15 +3312,22 @@ do_rpc_nodes () {
     local list=$1
     shift
 
-    do_nodesv $list "PATH=$LUSTRE/tests/:$PATH sh rpc.sh $@ " 
+    local RPATH="PATH=$LUSTRE/tests/:$PATH"
+    do_nodesv $list "${RPATH} NAME=${NAME} sh rpc.sh $@ "
 }
 
 wait_clients_import_state () {
     local list=$1
     local facet=$2
     local expected=$3
-    shift
 
+    local facets=$facet
+
+    if [ "$FAILURE_MODE" = HARD ]; then
+        facets=$(facets_on_host $(facet_active_host $facet))
+    fi
+
+    for facet in ${facets//,/ }; do
     local label=$(convert_facet2label $facet)
     local proc_path
     case $facet in
@@ -2937,8 +3335,10 @@ wait_clients_import_state () {
         mds* ) proc_path="mdc.$(get_clientmdc_proc_path $label).mds_server_uuid" ;;
         *) error "unknown facet!" ;;
     esac
+    local params=$(expand_list $params $proc_path)
+    done
 
-    if ! do_rpc_nodes $list wait_import_state $expected $proc_path; then
+    if ! do_rpc_nodes $list wait_import_state $expected $params; then
         error "import is not in ${expected} state"
         return 1
     fi
@@ -3081,3 +3481,67 @@ max_recovery_time () {
 
     echo $service_time 
 }
+
+remove_mdt_files() {
+    local facet=$1
+    local mdtdev=$2
+    shift 2
+    local files="$@"
+    local mntpt=${MOUNT%/*}/$facet
+
+    echo "removing files from $mdtdev on $facet: $files"
+    mount -t $FSTYPE $MDS_MOUNT_OPTS $mdtdev $mntpt || return $?
+    rc=0;
+    for f in $files; do
+	rm $mntpt/ROOT/$f || { rc=$?; break; }
+    done
+    umount -f $mntpt || return $?
+    return $rc
+}
+
+duplicate_mdt_files() {
+    local facet=$1
+    local mdtdev=$2
+    shift 2
+    local files="$@"
+    local mntpt=${MOUNT%/*}/$facet
+
+    echo "duplicating files on $mdtdev on $facet: $files"
+    mkdir -p $mntpt || return $?
+    mount -t $FSTYPE $MDS_MOUNT_OPTS $mdtdev $mntpt || return $?
+
+    do_umount() {
+	trap 0
+	popd > /dev/null
+	rm $tmp
+	umount -f $mntpt
+    }
+    trap do_umount EXIT
+
+    tmp=$(mktemp $TMP/setfattr.XXXXXXXXXX)
+    pushd $mntpt/ROOT > /dev/null || return $?
+    rc=0
+    for f in $files; do
+	touch $f.bad || return $?
+	getfattr -n trusted.lov $f | sed "s#$f#&.bad#" > $tmp
+	rc=${PIPESTATUS[0]}
+	[ $rc -eq 0 ] || return $rc
+	setfattr --restore $tmp || return $?
+    done
+    do_umount
+}
+
+run_sgpdd () {
+    local devs=${1//,/ }
+    shift
+    local params=$@
+    local rslt=$TMP/sgpdd_survey
+
+    # sgpdd-survey cleanups ${rslt}.* files
+
+    local cmd="rslt=$rslt $params scsidevs=\"$devs\" $SGPDDSURVEY"
+    echo + $cmd
+    eval $cmd
+    cat ${rslt}.detail
+}
+
