@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -131,11 +131,12 @@ command_t cmdlist[] = {
          "abort recovery on a restarting MDT or OST device\n"},
         {"set_timeout", jt_lcfg_set_timeout, 0,
          "usage: conf_param obd_timeout=<secs>\n"},
-        {"conf_param", jt_lcfg_mgsparam, 0, "set a permanent config param. "
+        {"conf_param", jt_lcfg_mgsparam, 0,"set a permanent config parameter.\n"
          "This command must be run on the MGS node\n"
-         "usage: conf_param <target.keyword=val> ...\n"},
+         "usage: conf_param [-d] <target.keyword=val>\n"
+         "  -d  Remove the permanent setting."},
         {"local_param", jt_lcfg_param, 0, "set a temporary, local param\n"
-         "usage: local_param <target.keyword=val> ...\n"},
+         "usage: local_param <target.keyword=val>\n"},
         {"get_param", jt_lcfg_getparam, 0, "get the Lustre or LNET parameter\n"
          "usage: get_param [-n | -N | -F] <param_path1 param_path2 ...> \n"
          "Get the value of Lustre or LNET parameter from the specified path.\n"
@@ -149,6 +150,13 @@ command_t cmdlist[] = {
          "usage: set_param [-n] <param_path1=value1 param_path2 value2 ...>\n"
          "Set the value of the Lustre or LNET parameter at the specified path\n"
          "  -n  Disable printing of the key name when printing values."},
+        {"list_param", jt_lcfg_listparam, 0,
+         "list the Lustre or LNET parameter name\n"
+         "usage: list_param [-FR] <param_path1 param_path2 ...>\n"
+         "List the name of Lustre or LNET parameter from the specified path.\n"
+         "  -F  Add '/', '@' or '=' for dirs, symlinks and writeable files,\n"
+                "respectively.\n"
+         "  -R  Recursively list all parameters under the specified path.\n"},
 
         /* Debug commands */
         {"==== debugging control ===", jt_noop, 0, "debug"},
