@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -105,24 +105,24 @@ command_t cmdlist[] = {
          "Create a new file with a specific striping pattern or\n"
          "set the default striping pattern on an existing directory or\n"
          "delete the default striping pattern from an existing directory\n"
-         "usage: setstripe [--size|-s stripe_size] [--offset|-o start_ost]\n"
-         "                 [--count|-c stripe_count] [--pool|-p <pool>]\n"
-         "                 <dir|filename>\n"
+         "usage: setstripe [--size|-s stripe_size] [--count|-c stripe_count]\n"
+	 "                 [--index|-i|--offset|-o start_ost_index]\n"
+         "                 [--pool|-p <pool>] <directory|filename>\n"
          "       or \n"
-         "       setstripe -d <dir>   (to delete default striping)\n"
+         "       setstripe -d <directory>   (to delete default striping)\n"
          "\tstripe_size:  Number of bytes on each OST (0 filesystem default)\n"
          "\t              Can be specified with k, m or g (in KB, MB and GB\n"
          "\t              respectively)\n"
-         "\tstart_ost:    OST index of first stripe (-1 filesystem default)\n"
+         "\tstart_ost_index: OST index of first stripe (-1 default)\n"
          "\tstripe_count: Number of OSTs to stripe over (0 default, -1 all)\n"
-         "\tpool:         Name of OST pool"},
+         "\tpool:         Name of OST pool to use (default none)"},
         {"getstripe", lfs_getstripe, 0,
          "To list the striping info for a given file or files in a\n"
          "directory or recursively for all files in a directory tree.\n"
          "usage: getstripe [--obd|-O <uuid>] [--quiet | -q] [--verbose | -v]\n"
-         "                 [--count | -c ] [--size | -s ] [--index | -i ]\n"
-         "                 [--offset | -o ] [--pool | -p ]\n"
-         "                 [--recursive | -r] <dir|file> ..."},
+         "                 [--count | -c ] [--index | -i | --offset | -o]\n"
+         "                 [--size | -s ] [--pool | -p ] [--directory | -d]\n"
+         "                 [--recursive | -r] <directory|filename> ..."},
         {"pool_list", lfs_poollist, 0,
          "List pools or pool OSTs\n"
          "usage: pool_list <fsname>[.<pool>] | <pathname>\n"},
@@ -150,7 +150,8 @@ command_t cmdlist[] = {
         {"join", lfs_join, 0,
          "join two lustre files into one - join A, B, will be like cat B >> A & del B\n"
          "usage: join <filename_A> <filename_B>\n"},
-        {"osts", lfs_osts, 0, "osts"},
+        {"osts", lfs_osts, 0, "list OSTs connected to client "
+         "[for specified path only]\n" "usage: osts [path]"},
         {"df", lfs_df, 0,
          "report filesystem disk space usage or inodes usage"
          "of each MDS and all OSDs or a batch belonging to a specific pool .\n"
@@ -326,7 +327,7 @@ static int lfs_setstripe(int argc, char **argv)
         }
         /* get the stripe offset */
         if (stripe_off_arg != NULL) {
-                st_offset = strtoul(stripe_off_arg, &end, 0);
+                st_offset = strtol(stripe_off_arg, &end, 0);
                 if (*end != '\0') {
                         fprintf(stderr, "error: %s: bad stripe offset '%s'\n",
                                 argv[0], stripe_off_arg);
@@ -454,7 +455,6 @@ static int lfs_find(int argc, char **argv)
         int c, ret;
         time_t t;
         struct find_param param = { .maxdepth = -1 };
-        char str[1024];
         struct option long_opts[] = {
                 /* New find options. */
                 {"atime",     required_argument, 0, 'A'},
@@ -536,29 +536,20 @@ static int lfs_find(int argc, char **argv)
                 case 'A':
                         xtime = &param.atime;
                         xsign = &param.asign;
+                        param.exclude_atime = !!neg_opt;
                 case 'C':
                         if (c == 'C') {
                                 xtime = &param.ctime;
                                 xsign = &param.csign;
+                                param.exclude_ctime = !!neg_opt;
                         }
                 case 'M':
                         if (c == 'M') {
                                 xtime = &param.mtime;
                                 xsign = &param.msign;
+                                param.exclude_mtime = !!neg_opt;
                         }
                         new_fashion = 1;
-                        if (neg_opt) {
-                                if (optarg[0] == '-')
-                                        optarg[0] = '+';
-                                else if (optarg[0] == '+')
-                                        optarg[0] = '-';
-                                else {
-                                        str[0] = '-';
-                                        str[1] = '\0';
-                                        strcat(str, optarg);
-                                        optarg = str;
-                                }
-                        }
                         ret = set_time(&t, xtime, optarg);
                         if (ret == INT_MAX)
                                 return -1;
@@ -689,18 +680,6 @@ static int lfs_find(int argc, char **argv)
                         };
                         break;
                 case 's':
-                        if (neg_opt) {
-                                if (optarg[0] == '-')
-                                        optarg[0] = '+';
-                                else if (optarg[0] == '+')
-                                        optarg[0] = '-';
-                                else {
-                                        str[0] = '-';
-                                        str[1] = '\0';
-                                        strcat(str, optarg);
-                                        optarg = str;
-                                }
-                        }
                         if (optarg[0] == '+')
                                 param.size_sign = -1;
                         else if (optarg[0] == '-')
@@ -715,6 +694,7 @@ static int lfs_find(int argc, char **argv)
                                         optarg);
                                 return ret;
                         }
+                        param.exclude_size = !!neg_opt;
                         param.size_check = 1;
                         break;
                 case 'v':
@@ -868,20 +848,33 @@ static int lfs_getstripe(int argc, char **argv)
 
 static int lfs_osts(int argc, char **argv)
 {
-        char mntdir[PATH_MAX] = {'\0'};
+        char mntdir[PATH_MAX] = {'\0'}, path[PATH_MAX] = {'\0'};
         struct find_param param;
         int index = 0, rc = 0;
 
-        if (argc != 1)
+        if (argc > 2)
                 return CMD_HELP;
 
-        while (llapi_search_mounts(NULL, index++, mntdir, NULL) == 0) {
+        if (argc == 2 && !realpath(argv[1], path)) {
+                rc = -errno;
+                fprintf(stderr, "error: invalid path '%s': %s\n",
+                        argv[1], strerror(-rc));
+                return rc;
+        }
+
+        while (!llapi_search_mounts(path, index++, mntdir, NULL)) {
+                /* Check if we have a mount point */
+                if (mntdir[0] == '\0')
+                        continue;
+
                 memset(&param, 0, sizeof(param));
                 rc = llapi_ostlist(mntdir, &param);
                 if (rc) {
                         fprintf(stderr, "error: %s: failed on %s\n",
                                 argv[0], mntdir);
                 }
+                if (path[0] != '\0')
+                        break;
                 memset(mntdir, 0, PATH_MAX);
         }
 
@@ -1094,16 +1087,15 @@ static int mntdf(char *mntdir, char *fsname, char *pool, int ishow, int cooked)
 
         printf("\n");
         showdf(mntdir, &sum, "filesystem summary:", ishow, cooked, NULL, 0,0);
-
+        printf("\n");
         return 0;
 }
 
 static int lfs_df(int argc, char **argv)
 {
-        char *path = NULL;
-        char *mntdir = NULL;
+        char mntdir[PATH_MAX] = {'\0'}, path[PATH_MAX] = {'\0'};
         int ishow = 0, cooked = 0;
-        int c, rc = 0;
+        int c, rc = 0, index = 0;
         char fsname[PATH_MAX] = "", *pool_name = NULL;
         struct option long_opts[] = {
                 {"pool", required_argument, 0, 'p'},
@@ -1126,45 +1118,26 @@ static int lfs_df(int argc, char **argv)
                         return CMD_HELP;
                 }
         }
-        if (optind < argc )
-                path = argv[optind];
 
-        if ((mntdir = malloc(PATH_MAX)) == NULL) {
-                fprintf(stderr, "error: cannot allocate %d bytes\n",
-                        PATH_MAX);
-                return -ENOMEM;
+        if (optind < argc && !realpath(argv[optind], path)) {
+                rc = -errno;
+                fprintf(stderr, "error: invalid path '%s': %s\n",
+                        argv[optind], strerror(-rc));
+                return rc;
         }
-        memset(mntdir, 0, PATH_MAX);
 
-        if (path) {
-                char rpath[PATH_MAX] = {'\0'};
+        while (!llapi_search_mounts(path, index++, mntdir, fsname)) {
+                /* Check if we have a mount point */
+                if (mntdir[0] == '\0')
+                        continue;
 
-                if (!realpath(path, rpath)) {
-                        rc = -errno;
-                        fprintf(stderr, "error: invalid path '%s': %s\n",
-                                path, strerror(-rc));
-                        return rc;
-                }
-
-                rc = llapi_search_mounts(rpath, 0, mntdir, fsname);
-                if (rc == 0 && mntdir[0] != '\0') {
-                        rc = mntdf(mntdir, fsname, pool_name, ishow, cooked);
-                        printf("\n");
-                }
-        } else {
-                int index = 0;
-
-                while (!llapi_search_mounts(NULL, index++, mntdir, fsname)) {
-                        rc = mntdf(mntdir, fsname, pool_name, ishow, cooked);
-                        if (rc)
-                                break;
-                        printf("\n");
-                        fsname[0] = '\0'; /* avoid matching in next loop */
-                        mntdir[0] = '\0'; /* avoid matching in next loop */
-                }
+                rc = mntdf(mntdir, fsname, pool_name, ishow, cooked);
+                if (rc || path[0] != '\0')
+                        break;
+                fsname[0] = '\0'; /* avoid matching in next loop */
+                mntdir[0] = '\0'; /* avoid matching in next loop */
         }
 
-        free(mntdir);
         return rc;
 }
 
@@ -2077,7 +2050,7 @@ static void print_quota(char *mnt, struct if_quotactl *qctl, int type)
                                         ? LPU64 : "["LPU64"]",
                                         dqb->dqb_bsoftlimit);
                         else
-                                sprintf(numbuf[1], "%s", "");
+                                sprintf(numbuf[1], "%s", "-");
                         sprintf(numbuf[2], (dqb->dqb_valid & QIF_BLIMITS)
                                 ? LPU64 : "["LPU64"]", dqb->dqb_bhardlimit);
                         printf(" %7s%c %6s %7s %7s",
@@ -2094,13 +2067,15 @@ static void print_quota(char *mnt, struct if_quotactl *qctl, int type)
                                         ? LPU64 : "["LPU64"]",
                                         dqb->dqb_isoftlimit);
                         else
-                                sprintf(numbuf[1], "%s", "");
+                                sprintf(numbuf[1], "%s", "-");
                         sprintf(numbuf[2], (dqb->dqb_valid & QIF_ILIMITS) ?
                                 LPU64 : "["LPU64"]", dqb->dqb_ihardlimit);
                         if (type != OST_QUOTA_INFO)
                                 printf(" %7s%c %6s %7s %7s",
                                        numbuf[0], iover ? '*' : ' ', numbuf[1],
                                        numbuf[2], iover > 1 ? timebuf : "-");
+                        else
+                                printf(" %7s %7s %7s %7s", "-", "-", "-", "-");
                         printf("\n");
                 }
         } else if (qctl->qc_cmd == LUSTRE_Q_GETINFO ||
