@@ -26,7 +26,7 @@
  * GPL HEADER END
  */
 /*
- * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
  * Use is subject to license terms.
  */
 /*
@@ -88,6 +88,7 @@
 #define INDEX_UNASSIGNED 0xFFFF
 #define MO_IS_LOOP     0x01
 #define MO_FORCEFORMAT 0x02
+#define min(x,y) ((x)<(y) ? (x) : (y))
 
 /* used to describe the options to format the lustre disk, not persistent */
 struct mkfs_opts {
@@ -517,7 +518,13 @@ static void enable_default_backfs_features(struct mkfs_opts *mop)
         int maj_high, maj_low, min;
         int ret;
 
-        strscat(mop->mo_mkfsopts, " -O dir_index,extents", sizeof(mop->mo_mkfsopts));
+	if (IS_OST(&mop->mo_ldd)) {
+        	strscat(mop->mo_mkfsopts, " -O dir_index,extents",
+			sizeof(mop->mo_mkfsopts));
+	} else {
+        	strscat(mop->mo_mkfsopts, " -O dir_index",
+			sizeof(mop->mo_mkfsopts));
+	}
 
         /* Upstream e2fsprogs called our uninit_groups feature uninit_bg,
          * check for both of them when testing e2fsprogs features. */
@@ -582,6 +589,11 @@ int make_lustre_backfs(struct mkfs_opts *mop)
                         return EINVAL;
                 }
                 block_count = mop->mo_device_sz / (L_BLOCK_SIZE >> 10);
+                /* If the LUN size is just over 2^32 blocks, limit the
+                 * filesystem size to 2^32-1 blocks to avoid problems with
+                 * ldiskfs/mkfs not handling this size.  Bug 22906 */
+                if (block_count > 0xffffffffULL && block_count < 0x100002000ULL)
+                        block_count = 0xffffffffULL;
         }
 
         if ((mop->mo_ldd.ldd_mount_type == LDD_MT_EXT3) ||
